
#include <CommandLineArgumentParser.h>
#include <sstream>
#include <algorithm>

#include "CLAFlagOption.h"
#include "CLAOptionWithParameter.h"

using namespace std;
using namespace CommandLineArgumentLib;

CommandLineArgumentParser::CommandLineArgumentParser() : valueIndex(0), helpFlagSet(false) {

}

CommandLineArgumentParser::CommandLineArgumentParser(const CommandLineArgumentParser& orig) {
}

CommandLineArgumentParser& CommandLineArgumentParser::operator=(const CommandLineArgumentParser &orig) {
}

CommandLineArgumentParser::~CommandLineArgumentParser() {
}

void CommandLineArgumentParser::AddHelpFlag(std::string helpFlag) {
    helpFlags.push_back(helpFlag);
}

void CommandLineArgumentParser::ParseArguments(int argc, char** argv) {
    ReadArguments(argc, argv);

    for (currentArgumentIterator = arguments.begin(); currentArgumentIterator != arguments.end(); currentArgumentIterator++) {
        string currentArgument = *currentArgumentIterator;

        if (ArgumentIsHelpFlag(currentArgument)) {
            helpFlagSet = true;
        } else if (IsOption(currentArgument)) {
            ParseOption();
        } else {
            ParseValue();
        }
    }

    if (!this->argumentManager.AllRequiredValuesWereFound() && !helpFlagSet) {
        throw CLAException("Could not find all required values");
    }
}

void CommandLineArgumentParser::ReadArguments(int argc, char** argv) {
    this->argumentCount = argc;

    for (int i = 0; i < argc; i++) {
        string argument(argv[i]);
        this->arguments.push_back(argument);
    }
}

bool CommandLineArgumentParser::IsOption(string argument) {
    return argument.at(0) == '-';
}

void CommandLineArgumentParser::ParseOption() {
    string currentArgument = *currentArgumentIterator;

    CLAOptionBase *option = argumentManager.GetArgumentType(currentArgument);

    CLAFlagOption *flagOption = dynamic_cast<CLAFlagOption*> (option);
    if (flagOption != NULL) {
        flagOption->Set();
        return;
    }

    CLAOptionWithParameter *parameterOption = dynamic_cast<CLAOptionWithParameter*> (option);
    if (parameterOption != NULL) {
        string nextArgument = GetNextArgument();

        parameterOption->Set(nextArgument);

        return;
    }

    stringstream ss;
    ss << currentArgument << " is no valid option.";
    throw CLAException(ss.str());
}

string CommandLineArgumentParser::GetNextArgument() {
    currentArgumentIterator++;

    if (currentArgumentIterator == arguments.end())
        throw CLAException("Could not retrieve parameter for option, end of argument list");

    return *currentArgumentIterator;
}

void CommandLineArgumentParser::ParseValue() {
    CLAValue *value = argumentManager.GetArgumentType(valueIndex);
    string currentArgument = *currentArgumentIterator;

    value->SetValue(currentArgument);

    valueIndex++;
}

bool CommandLineArgumentParser::ArgumentIsHelpFlag(string argument) {
    list<string>::iterator iter;

    iter = find(helpFlags.begin(), helpFlags.end(), argument);
    if (iter == helpFlags.end())
        return false;

    return true;
}

void CommandLineArgumentParser::AddArgumentType(CLArgumentBase* argumentType) {
    argumentManager.AddArgumentType(argumentType);
}

string CommandLineArgumentParser::GetOptionHelpMessage() {
    return argumentManager.GetOptionHelpMessage();
}
