/*
 * File:   ParseFlagOption_Test.cpp
 * Author: gruen
 *
 * Created on 29.12.2010, 14:30:12
 */

#include "ParseFlagOption_Test.h"

using namespace CommandLineArgumentLib;

CPPUNIT_TEST_SUITE_REGISTRATION(ParseFlagOption_Test);

ParseFlagOption_Test::ParseFlagOption_Test()
{
}

ParseFlagOption_Test::~ParseFlagOption_Test()
{
}

void ParseFlagOption_Test::setUp()
{
    parser = new CommandLineArgumentParser();
}

void ParseFlagOption_Test::tearDown()
{
    delete parser;
}

void ParseFlagOption_Test::SingleFlagOption_IsValid()
{
    //arrange
    CLAFlagOption option;
    SingleValueSetting<bool> *setting = dynamic_cast<SingleValueSetting<bool>* >(option.GetSetting());
    int argc = 1;
    char *argvar[1];

    option.SetIdentifier("-h");
    argvar[0] = "-h";

    parser->AddArgumentType(&option);

    //act
    parser->ParseArguments(argc, argvar);

    //assert
    CPPUNIT_ASSERT(setting->GetValue() == true);
}

void ParseFlagOption_Test::SingleFlagOption_NotValid()
{
    //arrange
    bool error = false;
    CLAFlagOption option;
    int argc = 1;
    char *argvar[1];

    option.SetIdentifier("-h");
    argvar[0] = "-a";

    parser->AddArgumentType(&option);

    //act
    try
    {
        parser->ParseArguments(argc, argvar);
    }
    catch (CLAException e)
    {
        error = true;
    }

    //assert
    CPPUNIT_ASSERT(error);
}

void ParseFlagOption_Test::MultipleFlagOptions_AllValid()
{
    //arrange
    CLAFlagOption o1, o2, o3;
    SingleValueSetting<bool> *s1 = dynamic_cast<SingleValueSetting<bool>* >(o1.GetSetting());
    SingleValueSetting<bool> *s2 = dynamic_cast<SingleValueSetting<bool>* >(o2.GetSetting());
    SingleValueSetting<bool> *s3 = dynamic_cast<SingleValueSetting<bool>* >(o3.GetSetting());
    int argc = 2;
    char *argvar[2];

    o1.SetIdentifier("-a");
    o2.SetIdentifier("-b");
    o3.SetIdentifier("-c");

    argvar[0] = "-c";
    argvar[1] = "-a";

    parser->AddArgumentType(&o1);
    parser->AddArgumentType(&o2);
    parser->AddArgumentType(&o3);

    //act
    parser->ParseArguments(argc, argvar);

    //assert
    CPPUNIT_ASSERT(s1->GetValue() == true);
    CPPUNIT_ASSERT(s2->GetValue() == false);
    CPPUNIT_ASSERT(s3->GetValue() == true);
}

void ParseFlagOption_Test::MultipleFlagOptions_OneNotValid()
{
    //arrange
    bool error = false;
    CLAFlagOption o1, o2;
    int argc = 3;
    char *argvar[3];

    o1.SetIdentifier("-a");
    o2.SetIdentifier("-b");

    argvar[0] = "-c";
    argvar[1] = "-a";
    argvar[2] = "-b";

    parser->AddArgumentType(&o1);
    parser->AddArgumentType(&o2);

    //act
    try
    {
        parser->ParseArguments(argc, argvar);
    }
    catch (CLAException e)
    {
        error = true;
    }

    //assert
    CPPUNIT_ASSERT(error == true);
}

