/*
 * File:   ParseParameterOption_Test.cpp
 * Author: gruen
 *
 * Created on 29.12.2010, 16:34:55
 */

#include "ParseParameterOption_Test.h"
#include <CLAParseArgumentFunctions.h>

using namespace CommandLineArgumentLib;

CPPUNIT_TEST_SUITE_REGISTRATION(ParseParameterOption_Test);

ParseParameterOption_Test::ParseParameterOption_Test()
{
}

ParseParameterOption_Test::~ParseParameterOption_Test()
{
}

void ParseParameterOption_Test::setUp()
{
    parser = new CommandLineArgumentParser();
}

void ParseParameterOption_Test::tearDown()
{
    delete parser;
}

void ParseParameterOption_Test::ParseSingleParameterOptionString_Success()
{
    //arrange
    CLAOptionWithParameter o;
    SingleValueSetting<std::string> s;
    int argc = 2;
    char *argv[2];

    s.ParseValue = &ParseArgumentFunctions::ParseString;
    o.SetSetting(&s);
    o.SetIdentifier("-a");

    argv[0] = "-a";
    argv[1] = "aParameterString";

    parser->AddArgumentType(&o);

    //act
    parser->ParseArguments(argc, argv);

    //assert
    CPPUNIT_ASSERT(s.IsValid() == true);
    CPPUNIT_ASSERT(s.GetValue() == argv[1]);
}

void ParseParameterOption_Test::ParseSingleParameterOptionInt_Success()
{
    //arrange
    CLAOptionWithParameter o;
    SingleValueSetting<int> s;
    int argc = 2;
    char *argv[2];

    s.ParseValue = &ParseArgumentFunctions::ParseNumber;
    o.SetSetting(&s);
    o.SetIdentifier("-a");

    argv[0] = "-a";
    argv[1] = "123";

    parser->AddArgumentType(&o);

    //act
    parser->ParseArguments(argc, argv);

    //assert
    CPPUNIT_ASSERT(s.IsValid() == true);
    CPPUNIT_ASSERT(s.GetValue() == 123);
}

void ParseParameterOption_Test::ParseSingleParameterOptionInt_ParameterInvalid()
{
    //arrange
    bool error = false;
    CLAOptionWithParameter o;
    SingleValueSetting<int> s;
    int argc = 2;
    char *argv[2];

    s.ParseValue = &ParseArgumentFunctions::ParseNumber;
    o.SetSetting(&s);
    o.SetIdentifier("-a");

    argv[0] = "-a";
    argv[1] = "1a23";

    parser->AddArgumentType(&o);

    //act
    try
    {
        parser->ParseArguments(argc, argv);
    }
    catch (CLAException e)
    {
        error = true;
    }

    //assert
    CPPUNIT_ASSERT(error == true);
}

void ParseParameterOption_Test::ParseSingleParameterOption_InvalidOption()
{
    //arrange
    bool error = false;
    CLAOptionWithParameter o;
    SingleValueSetting<int> s;
    int argc = 2;
    char *argv[2];

    s.ParseValue = &ParseArgumentFunctions::ParseNumber;
    o.SetSetting(&s);
    o.SetIdentifier("-a");

    argv[0] = "-b";
    argv[1] = "1a23";

    parser->AddArgumentType(&o);

    //act
    try
    {
        parser->ParseArguments(argc, argv);
    }
    catch (CLAException e)
    {
        error = true;
    }

    //assert
    CPPUNIT_ASSERT(error == true);
}

void ParseParameterOption_Test::ParseSingleParameterOption_NoParameter()
{
    //arrange
    bool error = false;
    CLAOptionWithParameter o;
    SingleValueSetting<int> s;
    int argc = 2;
    char *argv[2];

    s.ParseValue = &ParseArgumentFunctions::ParseNumber;
    o.SetSetting(&s);
    o.SetIdentifier("-a");

    argv[0] = "-a";

    parser->AddArgumentType(&o);

    //act
    try
    {
        parser->ParseArguments(argc, argv);
    }
    catch (CLAException e)
    {
        error = true;
    }

    //assert
    CPPUNIT_ASSERT(error == true);
}




