/* 
 * File:   CommandLineArgumentManager.h
 * Author: gruen
 *
 * Created on 29. Dezember 2010, 09:56
 */

#ifndef COMMANDLINEARGUMENTMANAGER_H
#define	COMMANDLINEARGUMENTMANAGER_H

#include <list>
#include <map>
#include <vector>
#include <string>
#include <iostream>

#include <CLAOptionBase.h>
#include <CLARequiredValue.h>
#include <CLAOptionalValue.h>
#include <CLAValue.h>
#include <CLAExceptions.h>

namespace CommandLineArgumentLib {

class CommandLineArgumentManager {

    class AppendOptionsHelpMessage : public std::unary_function<CLAOptionBase*, void> {
    public:
        void operator()(CLAOptionBase *option) {
            this->helpMessage += option->GetHelpMessage();
        }
        
        std::string GetHelpMessage() const {
            return this->helpMessage;
        }        

    private:
        std::string helpMessage;
    };

public:
    CommandLineArgumentManager();
    virtual ~CommandLineArgumentManager();

    void AddArgumentType(CLArgumentBase *argumentType);
    void AddArgumentType(CLAOptionBase *option);
    void AddArgumentType(CLARequiredValue *requiredValue);
    void AddArgumentType(CLAOptionalValue *optionalValue);

    CLAOptionBase* GetArgumentType(std::string optionIdentifier);
    CLAValue* GetArgumentType(int index);

    bool AllRequiredValuesWereFound();
    std::string GetOptionHelpMessage();
private:
    CommandLineArgumentManager(const CommandLineArgumentManager& orig);
    CommandLineArgumentManager & operator=(const CommandLineArgumentManager& orig);

    bool optionHelpMessageBuild;
    std::string optionHelpMessage;
    std::list<CLAOptionBase*> optionList;
    std::map<std::string, CLAOptionBase*> options;
    std::vector<CLARequiredValue*> requiredValues;
    std::vector<CLAOptionalValue*> optionalValues;
};
}

#endif	/* COMMANDLINEARGUMENTMANAGER_H */

