/* 
 * File:   SocketConnector.cpp
 * Author: jochen
 * 
 * Created on 11. Januar 2011, 16:13
 */

#include <SocketConnector.h>
#include <string>
#include <iostream>

using namespace std;
using namespace media_transport_protocol;

SocketConnector::SocketConnector() : receiverSocketList("Receiver Socket List"), senderSocketList("Sender Socket List") {
}

SocketConnector::SocketConnector(const SocketConnector& orig) : receiverSocketList("Receiver Socket List"), senderSocketList("Sender Socket List") {
}

SocketConnector::~SocketConnector() {
}

void SocketConnector::PrintSockets() {
    ostream &stream = controlCommunication->GetOutputStream();
    stream << "Receiving Sockets: " << endl;
    receiverSocketList.PrintSockets(stream);
    stream << endl << "Sending Sockets: " << endl;
    senderSocketList.PrintSockets(stream);
    stream << endl;
}

void SocketConnector::AddReceivingSocket(media_transport_protocol::prrt_socket* socket) {
    receiverSocketList.AddSocket(socket);
}

void SocketConnector::AddSendingSocket(media_transport_protocol::prrt_socket* socket) {
    senderSocketList.AddSocket(socket);
}

void SocketConnector::AddReceivingSocket(std::string senderIp, std::string receiverIp) {
    prrt_socket *socket = new prrt_socket();
    SocketInitialiser::InitReceiverSocket(*receiverSettings, *socket, senderIp, receiverIp);
    receiverSocketList.AddSocket(socket);
    ostream &stream = controlCommunication->GetOutputStream();
    stream << "Addet receiving socket from " << senderIp << " to " << receiverIp << endl;
}

void SocketConnector::AddSendingSocket(std::string senderIp, std::string receiverIp) {
    prrt_socket *socket = new prrt_socket();
    SocketInitialiser::InitSenderSocket(*senderSettings, *socket, senderIp, receiverIp);
    senderSocketList.AddSocket(socket);
    ostream &stream = controlCommunication->GetOutputStream();
    stream << "Addet sending socket from " << senderIp << " to " << receiverIp << endl;
}

void SocketConnector::ConnectSockets(int receiverSocketNo, int senderSocketNo) {

	  cerr << "##### ConnectSockets / Receiver Socket No: " << receiverSocketNo << endl;
	  cerr << "##### ConnectSockets / Sender Socket No: " << senderSocketNo << endl;
	
    prrt_socket *receiverSocket = receiverSocketList.GetSocket(receiverSocketNo);
    prrt_socket *senderSocket = senderSocketList.GetSocket(senderSocketNo);
    prrt_relay *newRelay = new prrt_relay();

    newRelay->SetSendingSocket(senderSocket);
    newRelay->SetReceivingSocket(receiverSocket);
    newRelay->SetRelayName(relaySettings->GetRelayName());

    relay->Stop();
    newRelay->Start();

    delete relay;
    relay = newRelay;

    ostream &stream = controlCommunication->GetOutputStream();
    stream << "Reroute from socket no. " << receiverSocketNo << " to socket no " << senderSocketNo << endl;
}

void SocketConnector::Shutdown() {
    if (relay) {
        relay->Stop();
        delete relay;
    }
    receiverSocketList.Shutdown();
    senderSocketList.Shutdown();
}

