/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include <sstream>

#include <ReceiveSettings.h>
#include <CLAParseArgumentFunctions.h>

using namespace CommandLineArgumentLib;
using namespace std;

ReceiveSettings::ReceiveSettings()
{
    senderAddressSetting.ParseValue = &ParseArgumentFunctions::ParseString;
    portSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    multicastAddressSetting.ParseValue = &ParseArgumentFunctions::ParseString;
    unicastAddressSetting.ParseValue = &ParseArgumentFunctions::ParseString;
    timeoutSetting.ParseValue = &ParseArgumentFunctions::ParseBool;
    bwEstSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    updateIntervalSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    feedbackSuppressionSetting.ParseValue = &ParseArgumentFunctions::ParseBool;
    maxFeedbackSentDelaySetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    delayCCSetting.ParseValue = &ParseArgumentFunctions::ParseString;

    portSetting.SetValue("5004");
    multicastAddressSetting.SetValue("224.0.2.2");
    timeoutSetting.SetValue("1");
    bwEstSetting.SetValue("0");
    updateIntervalSetting.SetValue("200");    // This setting isn't effected, if bandwidth estimation is disabled.
    feedbackSuppressionSetting.SetValue("0");
    maxFeedbackSentDelaySetting.SetValue("50");    // This setting isn't effected, if the feedback suppression is disabled.
    delayCCSetting.SetValue("3.0 5.0 0.5");    // corresponds to alpha, beta and gamma in delay-based congestion control equation.
}

ReceiveSettings::~ReceiveSettings()
{
}

string ReceiveSettings::ToString()
{
    stringstream ss;

    ss << "Sender Address: " << senderAddressSetting.GetValue() << endl;
    ss << "Port: " << portSetting.GetValue() << endl;
    ss << "Multicast Address: " << multicastAddressSetting.GetValue() << endl;
    ss << "Unicast Address: " << unicastAddressSetting.GetValue() << endl;
    ss << "Timeout active: " << timeoutSetting.GetValue() << endl;
    ss << "The Option of Bandwidth Estimation: " << bwEstSetting.GetValue() << endl;
    ss << "Update Interval of Bandwidth Estimation: " << updateIntervalSetting.GetValue() << endl;
    ss << "Feedback suppresion active: " << feedbackSuppressionSetting.GetValue() << endl;
    ss << "Maximum delay of sending feedback: " << maxFeedbackSentDelaySetting.GetValue() << endl;
    ss << "Delay-based congestion control (alpha beta gamma): " << delayCCSetting.GetValue() << endl;

    return ss.str();
}
