/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef RECEIVESETTINGS_H
#define	RECEIVESETTINGS_H

#include <string>
#include <typedefs.h>

class ReceiveSettings {
public:
    ReceiveSettings();
    ReceiveSettings(const ReceiveSettings& orig);
    virtual ~ReceiveSettings();

    bool IsUseMulticast() const {
        return !unicastAddressSetting.IsValid();
    }

    std::string GetSenderAddress() const {
        return senderAddressSetting.GetValue();
    }

    std::string GetTargetAddress() const {
        if(!IsUseMulticast())
            return unicastAddressSetting.GetValue();
        else
            return multicastAddressSetting.GetValue();
    }

    int GetPort() const {
        return portSetting.GetValue();
    }

    bool GetIsTimeoutActive() const {
        return timeoutSetting.GetValue();
    }

    bool GetIsFeedbackSuppressionActive() const {
        return feedbackSuppressionSetting.GetValue();
    }

    int GetBWESTMode() const {
        return bwEstSetting.GetValue();
    }

    int GetUpdateInterval() const {
        return updateIntervalSetting.GetValue();
    }

    int GetMaxFeedbackSentDelay() const {
        return maxFeedbackSentDelaySetting.GetValue();
    }

    std::string GetDelayCCParameters() const {
        return delayCCSetting.GetValue();
    }

    CommandLineArgumentLib::SettingBase* GetSenderAddressSetting() {
        return &this->senderAddressSetting;
    }

    CommandLineArgumentLib::SettingBase* GetPortSetting() {
        return &this->portSetting;
    }

    CommandLineArgumentLib::SettingBase* GetMulticastAddressSetting() {
        return &this->multicastAddressSetting;
    }

    CommandLineArgumentLib::SettingBase* GetUnicastAddressSetting() {
        return &this->unicastAddressSetting;
    }

    CommandLineArgumentLib::SettingBase* GetTimeoutSetting() {
        return &this->timeoutSetting;
    }

    CommandLineArgumentLib::SettingBase* GetFeedbackSuppressionSetting() {
        return &this->feedbackSuppressionSetting;
    }

    CommandLineArgumentLib::SettingBase* GetBWESTSetting() {
        return &this->bwEstSetting;
    }

    CommandLineArgumentLib::SettingBase* GetUpdateIntervalSetting() {
        return &this->updateIntervalSetting;
    }

    CommandLineArgumentLib::SettingBase* GetMaxFeedbackSentDelaySetting() {
        return &this->maxFeedbackSentDelaySetting;
    }

    CommandLineArgumentLib::SettingBase* GetDelayCCSetting() {
        return &this->delayCCSetting;
    }

    std::string ToString();
private:
    StringSetting senderAddressSetting;
    IntSetting portSetting;
    StringSetting multicastAddressSetting;
    StringSetting unicastAddressSetting;
    BoolSetting timeoutSetting;
    IntSetting bwEstSetting;
    IntSetting updateIntervalSetting;
    BoolSetting feedbackSuppressionSetting;
    IntSetting maxFeedbackSentDelaySetting;
    StringSetting delayCCSetting;
};

#endif	/* RECEIVESETTINGS_H */

