/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef ANALYSIS_H
#define ANALYSIS_H

//#define PKT_SIZE 1356
#define MAX_N 255 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <map>

#include "prrt_params.h"

namespace media_transport_protocol {

class prrt_analysis {
public:
    	prrt_analysis();

  	 ~prrt_analysis();

	param_AHEC get_AHEC(){
		return A_HEC;
	}


		double optimizeSingleParamSet(float Dtarget, float Ptarget, float CBw, float SBw, unsigned int R, float Pe, 
									float RTT, float Pf, float rho, unsigned int LP, float Dhost, float DTx, 
									float DPL, float DFB, float DSup);

  	double smoothPLR(double plr, double fall_acceleration, double max_deviation, double min_deviation);

    /**
     * This function computes the redundancy information, given 
     *
     * @num_recv the number of receiver.
     * @k        the number of data packets per block.
     * @n_p      the multipliers of parity packets vector. (array)
     * @n_rr     the number of parity packets rounds.
     * @plr      the packet loss rate.
     * @rho      the correlation coefficient of the Gilbert-Elliott channel model.
     */
	long double computeRI(unsigned int num_recv, unsigned int k, const unsigned int *n_p, const unsigned int n_rr, float plr, float rho);

private:

	param_AHEC A_HEC;
	double m_plr;

	class LineParam{
		
  	private:
		double slope;
		double intercept;
	public:
		LineParam (double _slope, double _intercept){
		 	slope=_slope;
			intercept=_intercept;
		};
		~LineParam (){}
		double get_slope(){
			return slope;		
		}
		double get_intercept(){
			return intercept;	
		}
        };	


	std::map<double, LineParam*> rho_0;       // map for storing slope and intercept values for rho=0.0
	std::map<double, LineParam*> rho_0_05;    // map for storing slope and intercept values for rho=0.0
	std::map<double, LineParam*> rho_0_1;   // map for storing slope and intercept values for rho=0.0
	


    	long double  Dist[MAX_N][MAX_N];
    	long double  FBdist[MAX_N][MAX_N];
		long double Factorials[MAX_N];

    	long double  factorial(int x);

	long double  Pdata(int n,int k,int i,int j);

	long double fastfactorial(unsigned int *num, unsigned int NUM_SIZE, unsigned int *denom, unsigned int DENOM_SIZE);	

	int          compare(const void *a, const void *b);

	void         bubblesort(unsigned int *tosort, unsigned int length);

	void		 initialize(void *array, unsigned int arraylen, long double value, unsigned int sizeType); 

	void 		 initialize2D(long double *array, long double value, unsigned int row, unsigned int col);

	long double  PLRperformance(unsigned int n, unsigned int k, float Pe, unsigned int *Np, unsigned int len, float Pf, unsigned int R, float rho);

	long double  Pdist(unsigned int d, unsigned int n, float Pe, const float rho);

	long double  RIperformance(const unsigned int R, const unsigned int n, const unsigned int k, const unsigned int *Np, const unsigned int r, const float Pe, const float rho);      

	void		 fill_num_denom(unsigned int *num, unsigned int NUM_SIZE, unsigned int *denom, unsigned int DENOM_SIZE, unsigned int n, unsigned int k, unsigned int i, unsigned int j);

	double 		 optimizeHEC(float Dtarget, float Ptarget, float CBw, float SBw, unsigned int R, float Pe, 
							float RTT, float Ts, float Ts_min, float Pf, float rho, unsigned int LP, 
							float Dhost, float DTx, float DPL, float DFB, float DSup);


  void         linearApproximateOptimumComputation(unsigned int Dtarget,unsigned int RTT, float Ts, int m, int& len_opt, 
			int& k_opt, int* Np_opt, long double& PLRres, long double& RIres, long double Ptarget, long double Pe, long double Pf, int R, double rho);


	void       directOptimumComputation (float Dtarget, float DTx, float Dhost, float Dresp, float DPL, float Ts, float RTT, unsigned int Nrr_max, unsigned int m, unsigned int& len_opt, unsigned int& k_opt, unsigned int* Np_opt, double& PLRres, double& RIres, float Ptarget, float Pe, float Pf, unsigned int R, float rho);

	void  		 FBdistribution(unsigned int *Np, unsigned int len, float Pf, unsigned int R);

	void		copyArray(unsigned int *Np, unsigned int *Np_opt, unsigned int len);

	void		 GEdistribution(float alpha, float beta, long double *PGE, unsigned int seq_len);

	int 	     approximate_n_linearly(long double rho, long double Pe, int k);


};  /*prrt_analysis*/

}
#endif /*ANALYSIS_H*/
