/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef DELAYCC_H
#define DELAYCC_H

#define MIN_RTT_SAMPLES 160
#define MAX_RTT_SAMPLES 32
#define PKT_SIZE 1356


namespace media_transport_protocol {

class prrt_delay_cc {
public:
    prrt_delay_cc();
    prrt_delay_cc(double alpha, double beta, double gamma);

    ~prrt_delay_cc();

    /**
     * @recv_rate : packets per second
     * return bandwidth estimation in kbits/s.
     */
    unsigned int calc_W(double recv_rate, double RTT, unsigned long lost_cnt);
    void setAlpha(double alpha);
    double getAlpha();
    void setBeta(double beta);
    double getBeta();
    void setGamma(double gamma);
    double getGamma();

private:

	void update_base_RTT(double RTT);

	double baseRTT;
	double maxRTT;
	
	//std::deque<float> minRTTs;

	struct value_pair {
		float value;
		unsigned int expiration;
  };

	value_pair* min_rtt;
	value_pair* max_rtt;
	value_pair* min_pair;
	value_pair* max_pair;
	value_pair* min_end;
	value_pair* min_last;
	value_pair* max_end;
	value_pair* max_last;

	unsigned int rtt_index;

	double m_gamma;     ///< Parameters of congestion control equation

  double m_beta;     ///< Parameters of congestion control equation

	double window;    ///< window size (in packets)

	double window_quant;

	double rate_quant;

	double m_alpha;     ///< Parameters of congestion control equation

  double m_avg_rtt;       ///< The average RTT by exponential smoothing. (in ms)
  int m_should_update;    ///< Indicates whether the window should be updated by the congestion control equation.
  unsigned long m_last_lost_cnt;    ///< The last loss counter.
  double m_last_avg_rtt;  ///< The last average RTT in which the window was updated by the congestion control equation. (in ms)
                          ///< i.e. The last average RTT to be acknowledged for the receiver data rate.
                          ///< used to convert the rate back to the window size.

};

};
#endif /*DELAYCC_H*/
