/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "error.h"

#include <sstream>

using namespace media_transport_protocol;
using namespace std;

prrt_error::prrt_error() {
}

prrt_error::prrt_error(const char *reason) {
    this->reason = reason;
}

prrt_error::~prrt_error() {
}

const char *prrt_error::what() {
    return reason.c_str();
}

//
// These are the functions for the error types that are defined for this class
//

socket_error::socket_error() {
}

socket_error::socket_error(const char* reason) {
    this->reason = reason;
}

module_error::module_error(const char* reason) {
    this->reason = reason;
}

module_error::module_error(const char* reason, int no) {
    this->reason = "";

    switch (no) {
        case -EGENERAL:
            this->reason = this->reason + reason + ", error when calling socket function!";
            break;
        case -EINVADDR:
            this->reason = this->reason + reason + ", invalid address given!";
            break;
        case -ENOSOCK:
            this->reason = this->reason + reason + ", socket not found!";
            break;
        case -EINVINP:
            this->reason = this->reason + reason + ", invalid input!";
            break;
        case -EINTERR:
            this->reason = this->reason + reason + ", interrupted!";
            break;
        case -EALLOC:
            this->reason = this->reason + reason + ", could not allocate memory!";
            break;
        default:
            stringstream errorStream;
            errorStream << this->reason << reason << ", unknown error (no " << no << ")!";
            this->reason = errorStream.str();
            break;
    }

    this->errorNumber = no;
}

list_error::list_error(const char* reason) {
    this->reason = reason;
}
