/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "prrt_params.h"

using namespace media_transport_protocol;


//
// param_PLR
//
param_PLR::param_PLR() {
	this->value = 0;
}

param_PLR::param_PLR(const std::string &val) {
	std::stringstream sstream(val);

	sstream >> this->value;
	if(sstream.fail())
		throw(value_error("Input value for param_PLR invalid"));
}

param_PLR::param_PLR(const int val) {
	value = val;
}

param_PLR::~param_PLR() {

}

std::string param_PLR::get_file_name() const {
	return std::string("plr");
}

std::string param_PLR::get_strvalue() const {
	std::string val;

	std::stringstream sstream;
	sstream << this->value;
	val = sstream.str();

	return val;
}


//
// param_AHEC
//
param_AHEC::param_AHEC()
: k(1)
, n_p()
, d_fec(20)
, d_c(2)
, d_req(40)
, d_fb(0)
, d_target(200)
{}

param_AHEC::param_AHEC(int _d_target, int _d_fec, int _d_c, int _d_req, int _d_fb, unsigned _k, const std::vector<unsigned> &_n_p)
: k(_k)
, n_p(_n_p)
, d_fec(_d_fec)
, d_c(_d_c)
, d_req(_d_req)
, d_fb(_d_fb)
, d_target(_d_target)
{
}

param_AHEC::~param_AHEC()
{}

std::string param_AHEC::get_file_name() const 
{
  return std::string("ahec");
}

std::string param_AHEC::get_strvalue() const
{
  std::string val;

  std::stringstream sstream;
  int n_rr = this->n_p.size();
  sstream << n_rr + 7 << " " << d_target << " " << d_fec << " " << d_c << " " << d_req << " " << d_fb << " " << k;

  std::vector<unsigned>::const_iterator iter;
  for(iter = n_p.begin(); iter != n_p.end(); ++iter) {
    sstream << " " << *iter;
  }

  val = sstream.str();

  return val;
}

void param_AHEC::set_k(unsigned _k)
{
  if (_k < 1) {
		throw(value_error("Input value for k of param_AHEC invalid"));
  }
  k = _k;
}

unsigned param_AHEC::get_k() const { return k; }

void param_AHEC::set_n_p(const std::vector<unsigned> &_n_p)
{
  for (int i=0; i< _n_p.size(); i++) {
    if (_n_p[i] < 0) {
		  throw(value_error("Input value for n_p of param_AHEC invalid"));
    }
  }
  n_p = _n_p;
}

std::vector<unsigned> param_AHEC::get_n_p() const { return n_p; }

void param_AHEC::set_n_p(const std::string &n_p)
{

		throw(value_error("Input value for n_p of param_AHEC invalid (string not support)"));
}

void param_AHEC::set_d_fec(int _d_fec)
{
  if (_d_fec < 0) {
		throw(value_error("Input value for d_fec of param_AHEC invalid"));
  }
  else if (_d_fec < 1) {
    // This case is ARQ.
    d_fec = 1;
  }
  else {
    d_fec = _d_fec;
  }
}

void param_AHEC::set_d_c(int _d_c)
{
  if (_d_c < 0) {
		throw(value_error("Input value for d_c of param_AHEC invalid"));
  }
  else if (_d_c < 1) {
    // This case is ARQ.
    d_c = 1;
  }
  else {
    d_c = _d_c;
  }
}

int param_AHEC::get_d_fec() const { return d_fec; }

int param_AHEC::get_d_c() const { return d_c; }

void param_AHEC::set_d_req(int _d_req)
{
  if (_d_req < 0) {
		throw(value_error("Input value for d_req of param_AHEC invalid"));
  }
  else if (_d_req < 10) {
    d_req = 10;
  }
  else {
    d_req = _d_req;
  }
}

int param_AHEC::get_d_req() const { return d_req; }

void param_AHEC::set_d_fb(int _d_fb)
{
  if (_d_fb < 0) {
		throw(value_error("Input value for d_fb of param_AHEC invalid"));
  }
  else {
    d_fb = _d_fb;
  }
}

int param_AHEC::get_d_fb() const { return d_fb; }

void param_AHEC::set_d_target(int _d_target)
{
  if (_d_target < 1) {
		throw(value_error("Input value for d_target of param_AHEC invalid"));
  }
  d_target = _d_target;
}

int param_AHEC::get_d_target() const { return d_target; }

void param_AHEC::add_n_p(unsigned last_n_p)
{
  if (last_n_p < 0) {
		throw(value_error("Input value for last_n_p of param_AHEC invalid"));
  }
  n_p.push_back(last_n_p);
}


//
// param_BW_est
//
param_BW_est::param_BW_est() {
	this->value = 0;
}

param_BW_est::param_BW_est(const std::string &val) {
	std::stringstream sstream(val);

	sstream >> this->value;
	if(sstream.fail())
		throw(value_error("Input value for param_BW_est invalid"));
}

param_BW_est::param_BW_est(const int val) {
	value = val;
}

param_BW_est::~param_BW_est() {

}

std::string param_BW_est::get_file_name() const {
	return std::string("bwest");
}

std::string param_BW_est::get_strvalue() const {
	std::string val;

	std::stringstream sstream;
	sstream << this->value;
	val = sstream.str();

	return val;
}


//
// param_BUF_fb
//
param_BUF_fb::param_BUF_fb() {
	this->value = 0;
}

param_BUF_fb::param_BUF_fb(const std::string &val) {
	std::stringstream sstream(val);

	sstream >> this->value;
	if(sstream.fail())
		throw(value_error("Input value for param_BUF_fb invalid"));
}

param_BUF_fb::param_BUF_fb(const int val) {
	value = val;
}

param_BUF_fb::~param_BUF_fb() {

}

std::string param_BUF_fb::get_file_name() const {
	return std::string("buffb");
}

std::string param_BUF_fb::get_strvalue() const {
	std::string val;

	std::stringstream sstream;
	sstream << this->value;
	val = sstream.str();

	return val;
}
