/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "prrt_relay.h"
#include "../prrt_module/kui/kui_error.h"
#include <sstream>

using namespace media_transport_protocol;

std::map<std::string, prrt_relay*> prrt_relay::relays;

prrt_relay::prrt_relay() {
    this->open = false;
    this->relayNo = -1;
}

prrt_relay::prrt_relay(const prrt_relay& orig) {
}

prrt_relay::~prrt_relay() {
    if (this->open)
        this->Stop();
}

void prrt_relay::Start()
throw (module_error, list_error, prrt_error) {
    StartRelay();
}

void prrt_relay::StartRelay()
throw (module_error, list_error, prrt_error) {
    std::string command;
    std::string feedback;
    int i = 0;
    int ret = 0;
    std::string register_file_path(PRRT_PATH + REGISTER_FILE);

    this->open = false;

    proc_file pf(register_file_path);

    std::stringstream commandStream;
    commandStream << "relay " << this->receivingSocket->sockno << " "
            << this->sendingSocket->sockno << " " << this->relayName;

    command = commandStream.str();

    pf.write(command);

    pf.read(feedback);

    /*the return value of the register file should be finished with an 'a'*/
    std::stringstream sstream(feedback);
    sstream >> ret;
    sstream.str("");
    sstream << ret;

    if (ret < 0) {
        throw (module_error("Could not create relay", ret));
    }

    sstream >> this->relayNo;
    this->open = true;
    relays.insert(std::make_pair(relayName, this));
    std::cerr << "Created relay no " << ret << std::endl;

    try {
    } catch (...) {
        this->Stop();
        throw (list_error("Could not insert socket in list!"));
    }
}

void prrt_relay::Stop() {
    std::string command;
    std::string feedback;
    int i = 0;
    int ret = 0;
    std::string register_file_path(PRRT_PATH + REGISTER_FILE);

    if (!this->open)
        return;

    proc_file pf(register_file_path);

    std::cerr << "Closing relay no " << this->relayNo << std::endl;

    std::stringstream commandStream;
    commandStream << "unrelay " << this->relayNo;

    command = commandStream.str();

    pf.write(command);

    pf.read(feedback);

    /*the return value of the register file should be finished with an 'a'*/
    std::stringstream sstream(feedback);
    sstream >> ret;

    this->open = false;
    relays.erase(relayName);
}

void prrt_relay::DeleteAllRelays() {
    while(relays.size() > 0) {
        prrt_relay *relay = relays.begin()->second;
        relays.erase(relay->GetRelayName());
        delete relay;
    }
}
