/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef PRRT_RELAY_H
#define	PRRT_RELAY_H

#include <string>
#include "prrt_sockets.h"
#include <map>

namespace media_transport_protocol {

class prrt_relay {
public:
    prrt_relay();
    prrt_relay(const prrt_relay& orig);

    virtual ~prrt_relay();

    // procedures
    void Start() throw (module_error, list_error, prrt_error);
    void Stop();

    // properties
    bool IsOpen() const {
        return open;
    }
    void SetSendingSocket(prrt_socket* sendingSocket) {
        this->sendingSocket = sendingSocket;
    }
    prrt_socket* GetSendingSocket() const {
        return sendingSocket;
    }
    void SetReceivingSocket(prrt_socket* receivingSocket) {
        this->receivingSocket = receivingSocket;
    }
    prrt_socket* GetReceivingSocket() const {
        return receivingSocket;
    }
    void SetRelayName(std::string relayName) {
        this->relayName = relayName;
    }
    std::string GetRelayName() const {
        return relayName;
    }

    static void DeleteAllRelays();

private:
    // functions
    void StartRelay() throw (module_error, list_error, prrt_error);

    // Fields
    prrt_socket *receivingSocket;   /// Socket that receives the data stream
    prrt_socket *sendingSocket;     /// Socket that forwards the received data stream
    bool open;                      /// Does the relay transport data between the sockets
    int relayNo;                    /// The number of the relay in the Kernel
    std::string relayName;          /// A name for the relay

    static std::map<std::string, prrt_relay*> relays; /// A map of all open relays (name, pointer pairs).
};

}

#endif	/* PRRT_RELAY_H */

