/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include <unistd.h>
#include "prrt_receiver.h"

using namespace media_transport_protocol;

prrt_receiver::prrt_receiver() {
}

prrt_receiver::prrt_receiver(std::string addr, unsigned int port, std::string path) {
	this->set_address(addr);
	this->set_fbaddress(addr);
	this->set_port(port);

	this->setup_socket();

	this->set_file(path);
}

prrt_receiver::prrt_receiver(std::string addr, std::string fbaddr, unsigned int port, std::string path) {
	this->set_address(addr);
	this->set_fbaddress(fbaddr);
	this->set_port(port);

	this->setup_socket();

	this->set_file(path);
}

prrt_receiver::~prrt_receiver() {

}

void prrt_receiver::set_address(std::string addr) {
	this->address = addr;
}

void prrt_receiver::set_fbaddress(std::string addr) {
	this->fbaddress = addr;
}

void prrt_receiver::set_port(unsigned int port) {
	this->port = port;
}

void prrt_receiver::setup_socket() {
	msock.bind(this->address, this->port, this->fbaddress);
}

void prrt_receiver::set_file(std::string path) {
	this->file_path = path;
}

void prrt_receiver::recv() {
	std::fstream stream_file(this->file_path.c_str(), std::ios_base::out);
	const int bufsize = 1500;
	char buffer[bufsize];
	int len = 0;

	try {
		while(true) {
			len = msock.recv(buffer, bufsize, 0);

			stream_file.write(buffer, len);
		}
	} catch (module_error &me) {
		std::cout << "Receiving abortet, " << me.what() << std::endl;
		stream_file.close();
	} catch (...) {
		stream_file.close();
		throw;
	}
}
