/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "kernel_wrappers.h"

#define DEBUG_NAME "KERNEL_WRAPPERS"

//void LogSending(void *data);
#if SIMULATION_ON_RECV
static int get_random_percent(void);
#endif

inline int wrap_kernel_sendmsg(struct socket *sock, struct sockaddr_in *peer, void *data, size_t data_len)
{
	int err = 0;
	struct kvec vec;

	INIT_MSGHDR(msg);

	if(sock == 0 || peer == 0 || data == 0) {
		PERROR("%s: Invalid arguments for wrap_kernel_sendmsg!\n", DEBUG_NAME);
		PERROR("%s: sock = %p, peer = %p, data = %p\n", DEBUG_NAME, sock, peer, data);
		return -1;
	}

	msg.msg_name = peer;
	msg.msg_namelen = sizeof(*peer);

	vec.iov_base = data;
	vec.iov_len = data_len;

// 	LogSending(data);

#if UNIT_TEST
	printk(KERN_EMERG "prrt, unit_test, kernel_wrapper: Sending packet\n");
#else
	err = kernel_sendmsg(sock, &msg, &vec, 1, data_len);
#endif

	return err;
}

//void LogSending(void *data)
//{
//	struct prrt_fbpacket *mfbpack = (struct prrt_fbpacket *)data;
//	struct prrt_packet_container *pcont = 0;
//
//	if(mfbpack->header.packtype == packtype_fb) {
//		prrt_fbpacket_ntoh(mfbpack);
//		prrt_fbpacket_log_sending(mfbpack);
//		prrt_fbpacket_hton(mfbpack);
//	} else {
//		pcont = container_of(data, struct prrt_packet_container, pack);
//		prrt_packet_ntoh(pcont);
//		prrt_packet_log_sending(pcont);
//		prrt_packet_hton(pcont);
//	}
//}
//
//void LogReceiving(void *data);

inline int wrap_kernel_recvmsg(struct socket *sock, void *data, size_t data_len)
{
	int err = 0;
	struct kvec vec;
#if SIMULATION_ON_RECV
	int dropped = 1;
#endif /*SIMULATION_ON_RECV*/

	INIT_MSGHDR(msg);

	if(sock == 0 || data == 0) {
		PERROR("%s: Invalid arguments for wrap_kernel_recvmsg!\n", DEBUG_NAME);
		PERROR("%s: sock = %p, data = %p\n", DEBUG_NAME, sock, data);
		return -1;
	}

	vec.iov_base = data;
	vec.iov_len = data_len;

#if UNIT_TEST
	printk(KERN_EMERG "prrt, unit_test, kernel_wrapper: Packet received\n");
#else
	#if SIMULATION_ON_RECV
	dropped = 1;
	while(dropped == 1) {
		PDEBUG(MSG_KERN_WRAP, "%s: Trying to receive packet\n", DEBUG_NAME);
		err = kernel_recvmsg(sock, &msg, &vec, 1, data_len, 0);
		if(err < 0)
			break;

		if(get_random_percent() > LOSS_RATE)
			dropped = 0;
		else {
			INIT_MSGHDR_MEMBERS(msg);
			bzero(vec.iov_base, vec.iov_len);
		}
	}
	#else /*SIMULATION_ON_RECV*/
	err = kernel_recvmsg(sock, &msg, &vec, 1, data_len, 0);
	#endif /*SIMULATION_ON_RECV*/
	
#endif

	//if(err == 0)
// 		LogReceiving(data);

	return err;
}

//void LogReceiving(void *data)
//{
//	struct prrt_fbpacket *mfbpack = (struct prrt_fbpacket *)data;
//	struct prrt_packet_container *pcont = 0;
//
//	if(mfbpack->header.packtype == packtype_fb) {
//		prrt_fbpacket_ntoh(mfbpack);
//		prrt_fbpacket_log_receiving(mfbpack);
//		prrt_fbpacket_hton(mfbpack);
//	} else {
//		pcont = container_of(data, struct prrt_packet_container, pack);
//		prrt_packet_ntoh(pcont);
//		prrt_packet_log_receiving(pcont);
//		prrt_packet_hton(pcont);
//	}
//}

/*
 * Returns a random number between 1 and 100.
 */
#if SIMULATION_ON_RECV
static int get_random_percent(void)
{
	u8 rnd = 0;
	get_random_bytes(&rnd, 1);
	return rnd*100/255+1;
}
#endif /*SIMULATION_ON_RECV*/
