/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "receiver_list.h"

#include <linux/slab.h>

#define DEBUG_NAME "RECEIVER_LIST"

void prrtrlist_init(struct prrt_receiver_list *rlist)
{
	INIT_LIST_HEAD(&rlist->head);
	init_MUTEX(&rlist->lock);

	prrtnpar_init_netparams(&rlist->worstNetParams);

	atomic_set(&rlist->count, 0);
}

int prrtrlist_reinit(struct prrt_receiver_list *rlist)
{
	int err = 0;

	prrtrlist_free(rlist);

	prrtnpar_reinit_netparams(&rlist->worstNetParams);

	atomic_set(&rlist->count, 0);

	return err;
}

void prrtrlist_free(struct prrt_receiver_list *rlist)
{
	struct list_head *iter = 0;
	struct list_head *next = 0;
	struct receiver_list_entry *p = 0;

	down(&rlist->lock);

	list_for_each_safe(iter, next, &rlist->head) {
		p = list_entry(iter, struct receiver_list_entry, list);

		list_del(iter);
		kfree(p);

		atomic_dec(&rlist->count);
	}

	up(&rlist->lock);
}

void prrtrlist_update_WorstNetParams(struct prrt_receiver_list *rlist)
{
	//struct receiver_list_entry *p = 0;
	//struct list_head *iter = 0;

	//int worst_t_replace = 0;
	//int worst_PLR = 0;

	//down(&rlist->lock);

	//list_for_each(iter, &rlist->head) {
	//	p = container_of(iter, struct receiver_list_entry, list);

	//	worst_PLR = MAX(worst_PLR, prrtnpar_get_PLR(&p->net_params));
	//	worst_t_replace = MAX(worst_t_replace, prrtnpar_get_ReplaceTime(&p->net_params));
	//}

	//up(&rlist->lock);

	//prrtnpar_set_ReplaceTime(&rlist->worstNetParams, worst_t_replace);
	//prrtnpar_set_PLR(&rlist->worstNetParams, worst_PLR);
}

static struct receiver_list_entry *add_receiver(struct prrt_receiver_list *rlist, u32 receiver)
{
	/*need to hold lock here*/
	struct receiver_list_entry *p = 0;

	/*allocate and initialise list entry*/
	p = kmalloc(sizeof(*p), GFP_KERNEL);
	p->recv_addr = receiver;
	prrtnpar_init_netparams(&p->net_params);

	list_add(&p->list, &rlist->head);

	atomic_inc(&rlist->count);

	return p;
}

static struct receiver_list_entry *get_receiver(struct prrt_receiver_list *rlist, u32 receiver)
{
	/*need to hold lock here*/
	struct list_head *iter = 0;
	struct receiver_list_entry *p = 0;

	/*search for receiver in list*/
	list_for_each(iter, &rlist->head) {
		p = list_entry(iter, struct receiver_list_entry, list);

		if(p->recv_addr == receiver)
			break;
	}

	if(iter == &rlist->head) {
		/*receiver was not found, we have to add it*/
		p = add_receiver(rlist, receiver);
	}

	return p;
}

void prrtrlist_update_ReplaceTime(struct prrt_receiver_list *rlist, u32 receiver, int sample_t_replace)
{
//	struct receiver_list_entry *p = 0;
//	int t_replace = 0;
//	int worst_t_replace = 0;
//
//	down(&rlist->lock);
//
//	p = get_receiver(rlist, receiver);
//
//	/*p is now a pointer to the list entry*/
//	prrtnpar_update_ReplaceTime(&p->net_params, sample_t_replace);
//
//	/*TODO:update worst RTT if necessary*/
//	worst_t_replace = prrtnpar_get_ReplaceTime(&rlist->worstNetParams);
//	t_replace = prrtnpar_get_ReplaceTime(&p->net_params);
//	if(t_replace > worst_t_replace)
//		prrtnpar_set_ReplaceTime(&rlist->worstNetParams, t_replace);
//
//	up(&rlist->lock);
}

void prrtrlist_setPLR(struct prrt_receiver_list *rlist, u32 receiver, int PLR)
{
	struct receiver_list_entry *p = 0;
	//int worst_plr = 0;

	down(&rlist->lock);

	p = get_receiver(rlist, receiver);

	/*p is now a pointer to the list entry*/
	prrtnpar_set_PLR(&p->net_params, PLR);

	/*TODO:update worst RTT if necessary*/
	//FIXME: find measure to determine worst PLR
	//worst_plr = prrtnpar_get_PLR(&rlist->worstNetParams);
	//if(PLR > worst_plr)
		prrtnpar_set_PLR(&rlist->worstNetParams, PLR);

	up(&rlist->lock);
}

void prrtrlist_setWorstTReplace(struct prrt_receiver_list *rlist, int t_replace)
{
	prrtnpar_set_ReplaceTime(&rlist->worstNetParams, t_replace);
}

void prrtrlist_print_stats(struct prrt_receiver_list *rlist) {
	PDEBUG(MSG_LISTSTATS, "%s: Number receivers currently registered: %d\n", DEBUG_NAME,
		atomic_read(&rlist->count));
}
