/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "loss_gatherer.h"
#include "defines.h"

#include "prrt_packet.h"
#include <linux/jiffies.h>
#include <linux/random.h>

#define DEBUG_NAME "LOSS_GATHERER"
/* TODO This value can be set by command line later. */
//#define MAX_FEEDBACK_SEND_DELAY 0

void prrt_lossgath_init(struct prrt_loss_gatherer *fbgath, struct prrt_time_state *time_state)
{
	INIT_LIST_HEAD(&fbgath->loss_list);
	init_MUTEX(&fbgath->lock);
	fbgath->count = 0;
	fbgath->max_fb_seqno = -1;
	fbgath->use_fb_limit = 0;
	fbgath->no_data_last_block = 0;
	fbgath->time_state = time_state;
  fbgath->max_fb_send_delay = 0;
}

void prrt_lossgath_free(struct prrt_loss_gatherer *fbgath)
{
	struct list_head *iter = 0;
	struct list_head *next = 0;
	struct fbgath_fb_entry *fbe = 0;

	/*free feebacks in the list*/
	down(&fbgath->lock);
	list_for_each_safe(iter, next, &fbgath->loss_list) {
		fbe = container_of(iter, struct fbgath_fb_entry, list);

		list_del(iter);

		kfree(fbe);
	}

	fbgath->count = 0;

	up(&fbgath->lock);
}

int prrt_lossgath_insertfb(struct prrt_loss_gatherer *fbgath, long packetno, u16 sn_base, u32 pack_timeout, u32 first_due, u32 replace_time)
{
  u16 rand = 0; /* A random number used for feedback sending delay. */
	struct fbgath_fb_entry *fbe = 0;
	//struct list_head *iter = 0;
	//struct list_head *next = 0;
	int fbcount = 0;
	u32 first_inserted = prrt_get_prrttime(fbgath->time_state);

//	if (TYPEII_HEC_MODE){
//		/*feedback already available for this block?*/
//		down(&fbgath->lock);
//		list_for_each_safe(iter, next, &fbgath->loss_list) {
//			fbe = container_of(iter, struct fbgath_fb_entry, list);
//
//	PDEBUG(MSG_FBADEB, "%s: looking for sn_base = %u, fbe->sn_base = %u\n", DEBUG_NAME, sn_base, fbe->sn_base );
//			if (fbe->sn_base == sn_base){
//				up(&fbgath->lock);
//				return 0;
//			}
//		}
//		up(&fbgath->lock);
//	}

	PDEBUG(PRINT_RECVWORK, "%s: Inserting loss of packet %ld\n", DEBUG_NAME, packetno);

	/*allocate new entry*/
	fbe = kmalloc(sizeof(*fbe), GFP_KERNEL);
	if(!fbe)
		return -ENOMEM;

	/*set variables in entry*/
	fbe->miss_packno = packetno;
	fbe->sn_base = sn_base;
	fbe->first_inserted = first_inserted;
	PDEBUG(MSG_FBADEB, "%s: first inserted = %u (packno %u)\n", DEBUG_NAME, fbe->first_inserted, fbe->miss_packno );
	fbe->retr_round = 1;	/*the 0th round is the transmission of the packet*/
	fbe->pack_timeout = pack_timeout;	/*the 0th round is the transmission of the packet*/
  get_random_bytes(&rand, sizeof(rand));
  fbe->fb_send_delay = fbgath->max_fb_send_delay * rand / 0xFFFF;
	fbe->first_due = first_due /*+ WORKDELAYMS + (replace_time >> 1)*/ + fbe->fb_send_delay;
	fbe->last_send = first_due /*+ WORKDELAYMS + (replace_time >> 1) + fbe->fb_send_delay*/;/*NOTE: important for new feedbacks: first_timestamp == timestamp*/
	PDEBUG(MSG_FBADEB, "%s: expired = %u\n", DEBUG_NAME, fbe->pack_timeout);
	//PERROR("%s: first inserted = %u (packno %u) expired= %u delay= %u max_delay= %u meanD=%d\n", DEBUG_NAME, fbe->first_inserted, fbe->miss_packno, fbe->pack_timeout, fbe->fb_send_delay, fbgath->max_fb_send_delay, atomic_read(&fbgath->time_state->mean_deviation));

	/*insert entry into list at the beginning because new feedbacks are due*/
	down(&fbgath->lock);
	list_add_tail(&fbe->list, &fbgath->loss_list);
	fbgath->count++;
	fbcount = fbgath->count;
	up(&fbgath->lock);
	//PDEBUG(MSG_FBADEB, "%s: Inserted feedback request for packet %ld, (count %d)\n", DEBUG_NAME, packetno, fbcount);
	//PDEBUGG(MSG_FBADEB, "%s: Estimated Sender Clock is %d\n", DEBUG_NAME, timestamp);

	return 0;
}

//static void SetMeasuredReplaceTimeIfInFirstRound(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe, u8 round, int* measured_t_replace);

int prrt_lossgath_delete1fb(struct prrt_loss_gatherer *fbgath, long packetno, u8 round, int *measured_t_replace)
{
	struct fbgath_fb_entry *fbe = 0;
	struct list_head *iter = 0;
	struct list_head *next = 0;
	int ret = -1;

	down(&fbgath->lock);

	list_for_each_safe(iter, next, &fbgath->loss_list) {
		fbe = container_of(iter, struct fbgath_fb_entry, list);

		if(fbe->miss_packno == packetno) {
			if (fbe->retr_round > 1){
				PDEBUG(MSG_FBADEB, "%s: Deleting feedback for packet %d\n", DEBUG_NAME, (int)packetno);
				/*found it: delete*/
				list_del(iter);

				//SetMeasuredReplaceTimeIfInFirstRound(fbgath, fbe, round, measured_t_replace);

				kfree(fbe);
				fbgath->count--;
			}
			ret = 0;
			goto out;
		}
	}

out:
	up(&fbgath->lock);

	return ret;
}

//static void SetMeasuredReplaceTimeIfInFirstRound(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe, u8 round, int* measured_t_replace)
//{
//	u32 now = prrt_get_prrttime(fbgath->time_state);
//	*measured_t_replace = 0;
//	//PERROR("Trying to set treplace for round %d", fbe->retr_round);
//
//	PDEBUG(MSG_FBADEB, "%s: Measuring replace time: fbe->retr_round %d, red->retr_round %d \n", DEBUG_NAME, fbe->retr_round, round);
//	if(round == 1) {
//		*measured_t_replace = prrt_diff_timestamps(fbgath->time_state, now, fbe->first_due);
//	//	*measured_t_replace = prrt_diff_timestamps(now, fbe->last_send);
//	PDEBUG(MSG_FBADEB, "%s: Estimated replace time %d\n", DEBUG_NAME, *measured_t_replace);
//	}
//}

int prrt_lossgath_deletefb(struct prrt_loss_gatherer *fbgath, long block_start, long block_end, u8 round,
				int *measured_t_replace)
{
	struct fbgath_fb_entry *fbe = 0;
	struct list_head *iter = 0;
	struct list_head *next = 0;
	int ret = -1;

	//PERROR("Trying to delete loss entry for block %ld - %ld", block_start, block_end);

	down(&fbgath->lock);

	list_for_each_safe(iter, next, &fbgath->loss_list) {
		fbe = container_of(iter, struct fbgath_fb_entry, list);

		if(prrt_packetno_lies_in_limits(fbe->miss_packno, block_start, block_end)) {
			if (fbe->retr_round > 1){
				PDEBUG(MSG_FBADEB, "%s: Deleting feedback for packet %d\n", DEBUG_NAME, fbe->miss_packno);
				/*found it: delete*/
				list_del(iter);

				//SetMeasuredReplaceTimeIfInFirstRound(fbgath, fbe, round, measured_t_replace);

				kfree(fbe);
				fbgath->count--;
			}
			ret = 0;
			goto out;
		}
	}

out:
	up(&fbgath->lock);

	return ret;
}

int prrt_lossgath_deleteAllFB(struct prrt_loss_gatherer *fbgath, long block_start, long block_end)
{
	struct fbgath_fb_entry *fbe = 0;
	struct list_head *iter = 0;
	struct list_head *next = 0;
	int ret = 0;

	down(&fbgath->lock);

	list_for_each_safe(iter, next, &fbgath->loss_list) {
		fbe = container_of(iter, struct fbgath_fb_entry, list);

		if(prrt_packetno_lies_in_limits(fbe->miss_packno, block_start, block_end)) {
			/*found it: delete*/
			list_del(iter);
			kfree(fbe);
			fbgath->count--;
		}
	}

	up(&fbgath->lock);

	return ret;
}

enum feedback_status_t {
	OLD,
	NOT_DUE,
	NOT_YET_IN_LIMIT,
	DUE,
	NOT_SET
};

static enum feedback_status_t getFeedbackStatus(struct prrt_loss_gatherer *fbgath,
						struct fbgath_fb_entry *fbe,
						u32 now, int replace_time, int D_target);
static void deleteOldEntry(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe);
static int entryIsOld(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe, u32 now, int replace_time, int D_target);

int prrt_lossgath_getnextfb(struct prrt_loss_gatherer *fbgath, u32 time_now, int replace_time, int D_target,
			  u16 *packno, u16 *retr_round, u16 *repetitions)
{
	struct fbgath_fb_entry *fbe = 0;
	struct list_head *iter = 0;
	struct list_head *use_as_head = &fbgath->loss_list;
	int ret = -1;
	enum feedback_status_t fbStatus = NOT_SET;
	u32 max_replace_time = MAX(5, replace_time);


	down(&fbgath->lock);

	if(list_empty(&fbgath->loss_list)){
		//PDEBUG(MSG_FBADEB, "%s: Feedback list empty\n", DEBUG_NAME);
		goto out;
	}

	/*get first entry*/
	iter = use_as_head->next;
	fbe = container_of(iter, struct fbgath_fb_entry, list);

	/*remove all entries that are too old*/
	while((fbStatus = getFeedbackStatus(fbgath, fbe, time_now, replace_time, D_target)) == OLD ||
			fbStatus == NOT_YET_IN_LIMIT || fbStatus == NOT_DUE) {

		//PDEBUG(MSG_FBADEB, "%s: Is feedback due for packet %d? (replace_time %d, at %u)\n", DEBUG_NAME, fbe->miss_packno, replace_time, time_now);

		if(fbStatus == OLD) {
			deleteOldEntry(fbgath, fbe);
		} else {
			use_as_head = iter;
			//PDEBUG(MSG_FBADEB, "%s: Feedback may not be sent yet(no %u)\n", DEBUG_NAME,
			//		fbe->miss_packno);
		}

		if(fbgath->count == 0){
			PDEBUG(MSG_FBADEB, "%s: Feedback count 0\n", DEBUG_NAME);
			goto out;
		}

		if(list_empty(&fbgath->loss_list)){
			PDEBUG(MSG_FBADEB, "%s: Feedback list empty\n", DEBUG_NAME);
			goto out; /*no feedbacks in list*/
		}


		iter = use_as_head->next;

		if(iter == &fbgath->loss_list){
			//PDEBUG(MSG_FBADEB, "%s: Reached beginning of feedback list\n", DEBUG_NAME);
			goto out; /*at the start of the list again*/
		}


		fbe = container_of(iter, struct fbgath_fb_entry, list);
	}

	/*decide whether there is an entry which is due and add it to end of list*/
	//PDEBUG(MSG_FBADEB, "%s: Checking feedback for packet %d on due-date\n", DEBUG_NAME,
			//fbe->miss_packno);
	if(fbStatus == DUE) {
		list_del_init(iter);

		*packno = fbe->miss_packno;

		if (fbe->retr_round > 1)
			fbe->last_send += max_replace_time;

		*retr_round = fbe->retr_round++; /*one transmission round more for this packet*/

		// entry is old if there is no time left for another retransmission (2 RTTs from now)
		if(entryIsOld(fbgath, fbe, 0, 2*max_replace_time, D_target)) {
			*repetitions = 3;
			PDEBUG(MSG_FBADEB, "%s: Early Old %d\n", DEBUG_NAME, fbe->miss_packno);
			deleteOldEntry(fbgath, fbe);
		}
		else{
			*repetitions = 1;
			//fbe->last_send = time_now;
			/*insert entry at the end of the list again*/
			list_add_tail(iter, &fbgath->loss_list);
		}
		ret = 0;
	}

out:
	up(&fbgath->lock);

	if(ret < 0) {
		//PDEBUG(MSG_FBADEB, "%s: No due feedback found\n", DEBUG_NAME);
	} else {
		PDEBUG(MSG_FBADEB, "%s: Feedback for packet %d is due at %u\n", DEBUG_NAME, *packno, time_now);
		//PDEBUG(MSG_FBADEB, "%s: Found loss of packet %d\n", DEBUG_NAME, *packno);
	}

	return ret;
}

static int entryIsDue(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe, u32 now, int replace_time);
static int entryIsNotYetInLimit(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe);

static enum feedback_status_t getFeedbackStatus(struct prrt_loss_gatherer *fbgath,
						struct fbgath_fb_entry *fbe,
						u32 now, int replace_time, int D_target) {
	u32 max_replace_time = MAX(5, replace_time);

	if(entryIsOld(fbgath, fbe, 0, max_replace_time, D_target)) {
		//PDEBUG(MSG_FBADEB, "%s: Old %d after round %u (at %u)\n", DEBUG_NAME, fbe->miss_packno, fbe->retr_round, now);
		return OLD;
	} else if(entryIsDue(fbgath, fbe, now, replace_time)) {
		if(entryIsNotYetInLimit(fbgath, fbe)){
		//PDEBUG(MSG_FBADEB, "%s: Not yet in limit %d\n", DEBUG_NAME, fbe->miss_packno);
			return NOT_YET_IN_LIMIT;
		}
		else{
		//PDEBUG(MSG_FBADEB, "%s: Due %d in round %u (replace_time %u, at %u)\n", DEBUG_NAME, fbe->miss_packno, fbe->retr_round, replace_time, now);
			return DUE;
		}
	} else {
		//PDEBUG(MSG_FBADEB, "%s: Not due %d\n", DEBUG_NAME, fbe->miss_packno);
		return NOT_DUE;
	}
}

/*
 * returns 1 if entry is too old, 0 if not
 */
//FIXME: parameter "now" is obsolete
static int entryIsOld(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe, u32 now, int replace_time, int D_target)
{
	if (fbe->retr_round > 1){
	u32 expiration_time = fbe->pack_timeout; //-replace_time;
	//u32 expected_replace_date = prrt_get_prrttime() + (u32)replace_time + WORKDELAYMS;
	u32 expected_replace_date = fbe->last_send + replace_time;

	//if(replace_time >= D_target) {
	//	/*if the time to replace a packet is bigger than the maximum allowed delay
	//	 *the entry is useless*/
	//	return 1;
	//} else {
	//	/*an entry is to old if there is no chance to send a replacement
	//	 *in RTT plus a workdelay so that it arrives in time(before it would
	//	 *be deleted from the list anyway)*/
	//	//FIXME: Need D_FEC and first_due here
	//	expiration_time = fbe->first_inserted+D_target-replace_time;
	//}

 	//PDEBUG(MSG_FBADEB, "%s: Comparing %u and %u\n", DEBUG_NAME, expected_replace_date, expiration_time);
	if(prrt_compare_timestamps(fbgath->time_state, expected_replace_date, expiration_time) >= 0)
		return 1;
	}

	return 0;
}

/*
 * returns 1 if entry is due, 0 if not
 */
static int entryIsDue(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe, u32 now, int replace_time)
{
	//u32 earliest_next_send = fbe->last_send + MAX(5, replace_time+WORKDELAYMS);
	// fb_send_delay for feedback suppression
	u32 earliest_next_send = fbe->last_send + MAX(5, replace_time) + fbe->fb_send_delay;
	/*a feedback is due if no replacement arrived in the RTT plus a workdelay*/

	//PDEBUG(MSG_FBADEB, "%s: Entry due?\n", DEBUG_NAME);
	//PDEBUG(MSG_FBADEB, "%s: first inserted = %u (packno %u)\n", DEBUG_NAME, fbe->first_inserted, fbe->miss_packno );
	//PDEBUG(MSG_FBADEB, "%s: expired = %u\n", DEBUG_NAME, fbe->pack_timeout );
	//PDEBUG(MSG_FBADEB, "%s: first due = %u\n", DEBUG_NAME, fbe->first_due);
	//PDEBUG(MSG_FBADEB, "%s: last send = %u\n", DEBUG_NAME, fbe->last_send);
	//PDEBUG(MSG_FBADEB, "%s: now = %u\n", DEBUG_NAME, now);
	//PDEBUG(MSG_FBADEB, "%s: earliest send time = %u\n", DEBUG_NAME, earliest_next_send);


	//PDEBUG(MSG_FBADEB, "%s: t_replace = %d\n", DEBUG_NAME, replace_time);

	/*new feedbacks are always due*/
	if(fbe->retr_round == 1){
		if(prrt_compare_timestamps(fbgath->time_state, now, fbe->first_due) >= 0){
			return 1;
		}
  }
	else {

		if(prrt_compare_timestamps(fbgath->time_state, now, earliest_next_send) >= 0){
			return 1;
		}
	}

	return 0;
}

static int entryIsNotYetInLimit(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe) {
	if(USE_FEEDBACK_STALLING && fbgath->use_fb_limit)
		return (prrt_comp_packetnos(fbe->miss_packno, fbgath->max_fb_seqno) > 0);
	else
		return 0;
}

static void deleteOldEntry(struct prrt_loss_gatherer *fbgath, struct fbgath_fb_entry *fbe) {
	/*delete entry and get next one*/
	PDEBUG(MSG_FBADEB, "%s: Removing old feedback for packet %d\n", DEBUG_NAME,
		fbe->miss_packno);
	list_del_init(&fbe->list);
	kfree(fbe);
	fbgath->count--;
}

void check_fb_limit_for_preredundancy(struct prrt_loss_gatherer *fbgath, struct prrt_packet_container *pcont);
void check_fb_limit_for_data(struct prrt_loss_gatherer *fbgath, struct prrt_packet_container *pcont);

void prrt_lossgath_check_fb_limit(struct prrt_loss_gatherer *fbgath, struct prrt_packet_container *pcont) {

	down(&fbgath->lock);

	if(prrt_packet_is_preredundancy(pcont)) {
		check_fb_limit_for_preredundancy(fbgath, pcont);
	} else if(prrt_packet_is_data(pcont) || prrt_packet_is_redata(pcont)) {
		check_fb_limit_for_data(fbgath, pcont);
	}

	//PDEBUG(MSG_FBSTALLING, "%s: New fb limit is: %ld (used=%d)\n", DEBUG_NAME,
	//	fbgath->max_fb_seqno, fbgath->use_fb_limit);

	up(&fbgath->lock);
}

void check_fb_limit_for_preredundancy(struct prrt_loss_gatherer *fbgath, struct prrt_packet_container *pcont) {
	int k = pcont->pack.red.red_header.k;
	u16 first_packno = pcont->pack.red.red_header.sn_base;
	u16 last_packno = prrt_increment_packetno(first_packno, k-1);

	fbgath->no_data_last_block = k;

	if(fbgath->use_fb_limit) {
		fbgath->max_fb_seqno = prrt_get_max_packetno(last_packno, fbgath->max_fb_seqno);
	} else {
		/*this is done directly because the packet number comparison does not work if
		  the last limit was not used for a long time*/
		fbgath->max_fb_seqno = last_packno;
		fbgath->use_fb_limit = 1;
	}
}

void check_fb_limit_for_data(struct prrt_loss_gatherer *fbgath, struct prrt_packet_container *pcont) {
	u16 packetno = pcont->pack.data.header.seqnum;
	u16 drop_limit_use_seqno = prrt_increment_packetno(fbgath->max_fb_seqno,
						MAYSENDFB_BLOCK_MULTI*fbgath->no_data_last_block);

	if(prrt_comp_packetnos(packetno, drop_limit_use_seqno) > 0) {
		fbgath->use_fb_limit = 0;
	}
}
