/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "param_local.h"

#define DEBUG_NAME "PARAM_LOCAL"

static const int delta = 3; /*division is implemented with rightshifts, this is the number
				 *which is used to divide when smoothing the results, e.g. 3 means
				 *division by 8*/

void prrtlpar_init_locparams(struct params_local *params)
{
    init_MUTEX(&params->lock);

    atomic_set(&params->D_target, D_TARGET_STARTMS);
    atomic_set(&params->D_Fec, D_FEC_STARTMS);
    atomic_set(&params->PLR_target, PLR_TARGET_START);
    atomic_set(&params->BW_est, BW_EST_START);
    atomic_set(&params->BUF_fb, 0);
    atomic_set(&params->Scheduled_RI, 0);
}

void prrtlpar_reinit_locparams(struct params_local *params)
{
    atomic_set(&params->D_target, D_TARGET_STARTMS);
    atomic_set(&params->D_Fec, D_FEC_STARTMS);
    atomic_set(&params->PLR_target, PLR_TARGET_START);
    atomic_set(&params->BW_est, BW_EST_START);
    atomic_set(&params->BUF_fb, 0);
    atomic_set(&params->Scheduled_RI, 0);
}

void prrtlpar_set_D_target(struct params_local *params, int d_target)
{
    atomic_set(&params->D_target, d_target);
}

int prrtlpar_get_D_target(struct params_local *params)
{
    return atomic_read(&params->D_target);
}

void prrtlpar_set_PLR_target(struct params_local *params, int plr_target)
{
    atomic_set(&params->PLR_target, plr_target);
}

int prrtlpar_get_PLR_target(struct params_local *params)
{
    return atomic_read(&params->PLR_target);
}

void prrtlpar_set_D_Fec(struct params_local *params, int d_fec)
{
    atomic_set(&params->D_Fec, d_fec);
}

int prrtlpar_get_D_Fec(struct params_local *params)
{
    return atomic_read(&params->D_Fec);
}

void prrtlpar_set_BW_est(struct params_local *params, int bw_est)
{
    atomic_set(&params->BW_est, bw_est);
}

int prrtlpar_get_BW_est(struct params_local *params)
{
    return atomic_read(&params->BW_est);
}

void prrtlpar_set_BUF_fb(struct params_local *params, int buf_fb)
{
    atomic_set(&params->BUF_fb, buf_fb);
}

int prrtlpar_get_BUF_fb(struct params_local *params)
{
    return atomic_read(&params->BUF_fb);
}

void prrtlpar_update_Scheduled_RI(struct params_local *params, int RI_increment)
{
//TODO: atomic_inc?
    atomic_set(&params->Scheduled_RI, RI_increment);
}

int prrtlpar_get_Scheduled_RI(struct params_local *params)
{
    return atomic_read(&params->Scheduled_RI);
}

