/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "param_network.h"

#define DEBUG_NAME "PARAM_NETWORK"

static const int delta = 3;	/*division is implemented with rightshifts, this is the number
				 *which is used to divide when smoothing the results, e.g. 3 means
				 *division by 8*/

void prrtnpar_init_netparams(struct params_network *params)
{
	int i = 0;

	init_MUTEX(&params->lock);

	down(&params->lock);

  for (i=0; i<MAX_MEASURES_RTT; i++) {
    params->rtt_samples[i] = 500;
  }
  params->rtt_samples_idx = 0;
  params->rtt_samples_sum = 500 * MAX_MEASURES_RTT;

	for(i=0; i<MAX_MEASURES_PLR; i++) {
//		params->bad_meas[i] = -1;
//		params->good_meas[i] = -1;
		params->loss_pattern[i] = 0;
		params->clock_offsets[i] = 0;
	}
	params->numberOfSamples = 0;	
	params->lastSampleIndex = 0;
//	params->last_bad_meas = -1;
//	params->last_good_meas = -1;
	params->alpha = -1;
	params->beta = -1;
	params->PLR = 0;
	
	params->numberOfGaps = 0;
	params->numberOfBursts = 0; 
	params->gapLength = 0;
	params->burstLength = 0;

	params->maxOffset = 0;
	params->minOffset = 0;
	params->lastOffset = 0;

	up(&params->lock);

	atomic_set(&params->t_replace, FIX_T_REPLACE);
	atomic_set(&params->fix_t_replace, FIX_T_REPLACE);
	atomic_set(&params->pack_interval, 0);
	atomic_set(&params->FTT, 0);
	atomic_set(&params->QueueDelay, 0);
}

void prrtnpar_reinit_netparams(struct params_network *params)
{
	int i = 0;

	down(&params->lock);

  for (i=0; i<MAX_MEASURES_RTT; i++) {
    params->rtt_samples[i] = 500;
  }
  params->rtt_samples_idx = 0;
  params->rtt_samples_sum = 500 * MAX_MEASURES_RTT;

	for(i=0; i<MAX_MEASURES_PLR; i++) {
//		params->bad_meas[i] = -1;
//		params->good_meas[i] = -1;
		params->loss_pattern[i] = 0;
		params->clock_offsets[i] = 0;
	}
	params->numberOfSamples = 0;	
	params->lastSampleIndex = 0;
//	params->last_bad_meas = -1;
//	params->last_good_meas = -1;
	params->alpha = -1;
	params->beta = -1;
	params->PLR = 0;

	params->maxOffset = 0;
	params->minOffset = 0;
	params->lastOffset = 0;

	up(&params->lock);

	atomic_set(&params->t_replace, 0);
	atomic_set(&params->fix_t_replace, FIX_T_REPLACE);
	atomic_set(&params->pack_interval, 0);
	atomic_set(&params->FTT, 0);
	atomic_set(&params->QueueDelay, 0);
}

void prrtnpar_update_ReplaceTime(struct params_network *params, int sample_t_replace)
{
	int t_replace = atomic_read(&params->t_replace);
	int diff = sample_t_replace - t_replace;

	PDEBUG(MSG_FINAL, "%s: Sample replace time = %d\n", DEBUG_NAME, sample_t_replace);
	PDEBUG(MSG_PARAM, "%s: Old replace time = %d\n", DEBUG_NAME, t_replace);

	//if(t_replace == 0) {
	//	t_replace = sample_t_replace;
	//	atomic_set(&params->t_replace, t_replace);
	//	return;
	//}

	//if (diff > t_replace)
	//return;

	//t_replace += (int)(diff >> delta);
  if (diff > -1000 && diff < 1000) {
	  //t_replace = (int)((7*t_replace + sample_t_replace) >> delta) + 1;   // +1 => obtain at least ceil.
    params->rtt_samples_sum += sample_t_replace - params->rtt_samples[params->rtt_samples_idx];
    t_replace = params->rtt_samples_sum / MAX_MEASURES_RTT;
    t_replace++;
    params->rtt_samples[params->rtt_samples_idx] = sample_t_replace;
    params->rtt_samples_idx = (params->rtt_samples_idx + 1) % MAX_MEASURES_RTT;
  }

	//t_replace = (t_replace <= 0 ? 1000 : t_replace);

	PDEBUG(MSG_PARAM, "%s: New replace time = %d\n", DEBUG_NAME, t_replace);
	atomic_set(&params->t_replace, t_replace);
}

int prrtnpar_set_ReplaceTime(struct params_network *params, int t_replace)
{
	atomic_set(&params->t_replace, t_replace);
	atomic_set(&params->fix_t_replace, t_replace);

	PDEBUG(MSG_PARAM, "%s: Replace time set to %d\n", DEBUG_NAME, t_replace);

	return 0;
}

int prrtnpar_get_ReplaceTime(struct params_network *params)
{
	if(USE_FIXED_T_REPLACE)
		return atomic_read(&params->fix_t_replace);
	else
		return atomic_read(&params->t_replace);
	
}

void prrtnpar_update_PLR(struct params_network *params)
{
	int i = 0;
	u32 numberOfLosses = 0;
	u32 numberOfGaps = 1 - params->loss_pattern[0];
	u32 gaplength = numberOfGaps;
	u32 numberOfBursts = params->loss_pattern[0];
	u32 burstlength = numberOfBursts;
	u8 stateBad = params->loss_pattern[0];

	down(&params->lock);

	for(i=0; i<MAX_MEASURES_PLR; i++) {
		numberOfLosses += params->loss_pattern[i];

		// in bad state and next packet lost
		if (stateBad && params->loss_pattern[i]){
			burstlength++;
		}
		// in bad state and next packet received 
		else if (stateBad && !params->loss_pattern[i]){
			stateBad = 0;

			numberOfGaps++;
			gaplength++;
		}
		// in good state and next packet lost
		else if (!stateBad && params->loss_pattern[i]){
			stateBad = 1;

			numberOfBursts++;
			burstlength++;
		}
		// in good state and next packet received 
		else{
			gaplength++;
		}
	}
	
	numberOfLosses *= MAX_PARAM;
	params->PLR = numberOfLosses/MAX_MEASURES_PLR;

	params->numberOfGaps = numberOfGaps;
	params->numberOfBursts = numberOfBursts;
	params->gapLength = gaplength;
	params->burstLength = burstlength;

	up(&params->lock);
}

//void prrtnpar_update_PLR(struct params_network *params)
//{
//	int numberOfMeasuredPackets = 0;
//	int numberMeasurements = 0;
//	int i = 0;
//	int alpha = 0;
//	int beta = 0;
//	int plr = 0;
//
//	if(params->last_good_meas == -1 || params->last_bad_meas == -1)
//		return;
//
//	down(&params->lock);
//
//	/*update beta*/
//	/*determine the sum of all received packets in the measurements and the number of
//	 *measurements*/
//	numberOfMeasuredPackets = 0;
//	numberMeasurements=0;
//	i=params->last_good_meas;
//	while(numberMeasurements<MAX_MEASURES_PLR) {
//		if(params->good_meas[i] == -1)
//			break;
//
//		numberOfMeasuredPackets += params->good_meas[i];
//		i--;
//		i %= MAX_MEASURES_PLR;
//		if(i<0) {
//			i += MAX_MEASURES_PLR;
//		}
//		numberMeasurements++;
//	}
//
//	if(numberOfMeasuredPackets == 0) {
//		/*in this case we haven't received any packets yet so we set PLR to 0%*/
//		beta = MAX_PARAM;
//		beta = 0;
//		plr = 0;
//		goto out;
//	}
//
//	/*this is in percent*/
//	beta = MAX_PARAM;
//	if(numberOfMeasuredPackets != 0)
//		beta -= numberMeasurements*MAX_PARAM/numberOfMeasuredPackets;
//
//	PDEBUG(MSG_PARAM, "%s: Beta set to %d\n", DEBUG_NAME, beta);
//
//	/*update alpha*/
//	/*determine the sum of all not received packets in the measurements and the number of
//	 *measurements*/
//	numberOfMeasuredPackets = 0;
//	numberMeasurements=0;
//	i=params->last_bad_meas;
//	while(numberMeasurements<MAX_MEASURES_PLR) {
//		if(params->bad_meas[i] == -1)
//			break;
//
//		numberOfMeasuredPackets += params->bad_meas[i];
//		i--; i %= MAX_MEASURES_PLR;
//		if(i<0) {
//			i += MAX_MEASURES_PLR;
//		}
//		numberMeasurements++;
//	}
//
//	/*this is in percent*/
//	alpha = MAX_PARAM;
//	if(numberOfMeasuredPackets != 0)
//		alpha -= numberMeasurements*MAX_PARAM/numberOfMeasuredPackets;
//
//	PDEBUG(MSG_PARAM, "%s: Beta set to %d\n", DEBUG_NAME, alpha);
//
//	if(MAX_PARAM-alpha == 0) {
//		plr = 0;
//		goto out;
//	}
//	//if((MAX_PARAM+MAX_PARAM*(MAX_PARAM-beta)) == 0) {
//	if((MAX_PARAM-alpha+MAX_PARAM-beta) == 0) {
//		PERROR("%s: Could not estimate PLR, division by zero(MAX_PARAM+MAX_PARAM*(MAX_PARAM-beta))\n", DEBUG_NAME);
//		/*do not change anything*/
//		alpha = params->alpha;
//		beta = params->beta;
//		plr = params->PLR;
//		goto out;
//	}
//
//	/*update PLR(in percent)
//	 *this must be upscaled because we can only use natural numbers in kernel space*/
//	//plr = 10000/(MAX_PARAM+MAX_PARAM*(MAX_PARAM-beta)/(MAX_PARAM-alpha));
//	plr = (MAX_PARAM-beta)*MAX_PARAM/(MAX_PARAM-alpha+MAX_PARAM-beta);
//
//out:
//	params->alpha = alpha;
//	params->beta = beta;
//	params->PLR = plr;
//	PDEBUG(PRINT_PARAMS, "%s: PLR set to %d\n", DEBUG_NAME, params->PLR);
//
//	up(&params->lock);
//}

int prrtnpar_set_PLR(struct params_network *params, int PLR)
{
	long long new_plr = 0;

	if(PLR < 0 || PLR > MAX_PARAM)
		return -1;

	down(&params->lock);
	
	//TODO:
	new_plr = 15*params->PLR+PLR;

	params->PLR = (int)(new_plr >> 4);

	up(&params->lock);

	PDEBUG(MSG_PARAM, "%s: PLR set to %d\n", DEBUG_NAME, PLR);

	return 0;
}

void prrtnpar_insert_Loss(struct params_network *params, u16 seqnum)
{
	down(&params->lock);
	
	params->loss_pattern[seqnum%MAX_MEASURES_PLR] = 1;

	up(&params->lock);
}

void prrtnpar_insert_Sent(struct params_network *params, u16 seqnum)
{
	down(&params->lock);

	params->lastSampleIndex = seqnum%MAX_MEASURES_PLR;
	params->loss_pattern[params->lastSampleIndex] = 0;
	
	if (params->numberOfSamples < MAX_MEASURES_PLR)
		params->numberOfSamples++;

	up(&params->lock);
}

void prrtnpar_get_LossPattern(struct params_network *params, char* pattern)
{
	int index = 0;
	u16 startIndex = 0;

	down(&params->lock);
	startIndex = ((MAX_MEASURES_PLR - params->numberOfSamples) + params->lastSampleIndex)%MAX_MEASURES_PLR;

	for (index = 0; index < (params->numberOfSamples - 1); index++)
	//for (index = 0; index < (MAX_MEASURES_PLR - 1); index++)
	{
		if (params->loss_pattern[(startIndex + index)%MAX_MEASURES_PLR])
		//if (params->loss_pattern[(index)%MAX_MEASURES_PLR])
		pattern[index] = '1';
		else
		pattern[index] = '0';
	}

	pattern[index] = 0;

	params->numberOfSamples = 0;

	up(&params->lock);
}

//void prrtnpar_insert_GoodMeas(struct params_network *params, int meas)
//{
//	down(&params->lock);
//
//	if(params->last_good_meas == -1) {
//		/*this is the first measurement*/
//		params->good_meas[0] = meas;
//		params->last_good_meas = 0;
//	} else {
//		/*not the first one
//		 *move pointer for last entry and update value*/
//		params->last_good_meas++;
//		params->last_good_meas %= MAX_MEASURES_PLR;
//
//		params->good_meas[params->last_good_meas] = meas;
//	}
//
//	PDEBUG(MSG_PARAM, "%s: Inserted good measurement %d at position %d\n", DEBUG_NAME, meas,
//		params->last_good_meas);
//
//	up(&params->lock);
//}
//
//
//void prrtnpar_update_last_GoodMeas(struct params_network *params)
//{
//	/*no measurements taken yet*/
//	if(params->last_good_meas == -1)
//		return;
//
//	down(&params->lock);
//
//	params->good_meas[params->last_good_meas] += 1;
//
//	PDEBUG(MSG_PARAM, "%s: Updated good measurement at position %d to %d\n", DEBUG_NAME,
//		params->last_good_meas, params->good_meas[params->last_good_meas]);
//
//	up(&params->lock);
//}
//
//void prrtnpar_insert_BadMeas(struct params_network *params, int meas)
//{
//	down(&params->lock);
//
//	if(params->last_bad_meas == -1) {
//		/*this is the first measurement*/
//		params->bad_meas[0] = meas;
//		params->last_bad_meas = 0;
//	} else {
//		/*not the first one
//		 *move pointer for last entry and update value*/
//		params->last_bad_meas++;
//		params->last_bad_meas %= MAX_MEASURES_PLR;
//
//		params->bad_meas[params->last_bad_meas] = meas;
//	}
//
//	PDEBUG(MSG_PARAM, "%s: Inserted bad measurement %d at position %d\n", DEBUG_NAME, meas,
//		params->last_bad_meas);
//
//	up(&params->lock);
//}

int prrtnpar_get_PLR(struct params_network *params)
{
	int plr;

	down(&params->lock);

	plr = params->PLR;

	up(&params->lock);

	return plr;
}

void prrtnpar_set_Alpha(struct params_network *params, int alpha)
{
	down(&params->lock);

	params->alpha = alpha;

	up(&params->lock);
}

int prrtnpar_get_Alpha(struct params_network *params)
{
	int alpha;

	down(&params->lock);

	alpha = params->alpha;

	up(&params->lock);

	return alpha;
}

void prrtnpar_set_Beta(struct params_network *params, int beta)
{
	down(&params->lock);

	params->beta = beta;

	up(&params->lock);
}

int prrtnpar_get_Beta(struct params_network *params)
{
	int beta;

	down(&params->lock);

	beta = params->beta;

	up(&params->lock);

	return beta;
}

void prrtnpar_update_PacketInterval(struct params_network *params, int sample_pack_interval)
{
    int pack_interval = atomic_read(&params->pack_interval);
    //int diff = sample_pack_interval - pack_interval;
    int smoothed_diff = 0;

    //PDEBUG(MSG_OUTLIST, "%s: Sample gathering time = %d\n", DEBUG_NAME, sample_pack_interval);
    //PDEBUG(MSG_OUTLIST, "%s: Old gathering time = %d\n", DEBUG_NAME, pack_interval);

    if (pack_interval == 0)
    {
        pack_interval = sample_pack_interval;
        atomic_set(&params->pack_interval, pack_interval);
        return;
    }

    //	PDEBUG(MSG_OUTLIST, "%s: Smoothed diff = %d\n", DEBUG_NAME, (int)(diff >> delta));
    //	smoothed_diff = (int)(diff >> delta);
    //	smoothed_diff = (smoothed_diff < 0 ? smoothed_diff+1 : smoothed_diff);
    //	pack_interval += smoothed_diff;
    //	PDEBUG(MSG_OUTLIST, "%s: Smoothed diff = %d\n", DEBUG_NAME, smoothed_diff);
    smoothed_diff = (31 * pack_interval + 1 * sample_pack_interval);
    pack_interval = (int) (smoothed_diff >> 5);
    pack_interval = (pack_interval <= 0 ? 1000 : pack_interval);
    //pack_interval = 4;
    //PDEBUG(MSG_OUTLIST, "%s: New gathering time = %d\n", DEBUG_NAME, pack_interval);
    atomic_set(&params->pack_interval, pack_interval);
}

int prrtnpar_get_PacketInterval(struct params_network *params)
{
    return atomic_read(&params->pack_interval);
}

void prrtnpar_set_NumGaps(struct params_network *params, int numGaps)
{
	down(&params->lock);

	params->numberOfGaps = numGaps;

	up(&params->lock);
}

void prrtnpar_set_NumBursts(struct params_network *params, int numBursts)
{
	down(&params->lock);

	params->numberOfBursts = numBursts;

	up(&params->lock);
}

void prrtnpar_set_GapLength(struct params_network *params, int gapLength)
{
	down(&params->lock);

	params->gapLength = gapLength;

	up(&params->lock);
}

void prrtnpar_set_BurstLength(struct params_network *params, int burstLength)
{
	down(&params->lock);

	params->burstLength = burstLength;

	up(&params->lock);
}

int prrtnpar_get_NumGaps(struct params_network *params)
{
	int numGaps;

	down(&params->lock);

	numGaps = params->numberOfGaps;

	up(&params->lock);

	return numGaps;
}

int prrtnpar_get_NumBursts(struct params_network *params)
{
	int numBursts;

	down(&params->lock);

	numBursts = params->numberOfBursts;

	up(&params->lock);

	return numBursts;
}

int prrtnpar_get_BurstLength(struct params_network *params)
{
	int burstLength;

	down(&params->lock);

	burstLength = params->burstLength;

	up(&params->lock);

	return burstLength;
}

int prrtnpar_get_GapLength(struct params_network *params)
{
	int gapLength;

	down(&params->lock);

	gapLength = params->gapLength;

	up(&params->lock);

	return gapLength;
}

void prrtnpar_update_FTT(struct params_network *params, int FTT_sample)
{
	atomic_set(&params->FTT, FTT_sample);
}

int prrtnpar_get_FTT(struct params_network *params)
{
	return atomic_read(&params->FTT);
}

void prrtnpar_update_QueueDelay(struct params_network *params, int QueueDelay_sample, int seqnum)
{
	down(&params->lock);

	params->clock_offsets[seqnum%MAX_MEASURES_PLR] = QueueDelay_sample;
	params->lastOffset = QueueDelay_sample;
	
	up(&params->lock);
}

int prrtnpar_get_QueueDelay(struct params_network *params)
{
	int queue_delay = 0;
	int i = 0;

	down(&params->lock);

	params->maxOffset = INT_MIN;
	params->minOffset = INT_MAX;

	for(i=0; i<MAX_MEASURES_PLR; i++) {
		if (params->clock_offsets[i] > params->maxOffset)
			params->maxOffset = params->clock_offsets[i];
		else if (params->clock_offsets[i] < params->minOffset)
			params->minOffset = params->clock_offsets[i];
	}

	queue_delay = params->lastOffset - params->minOffset;

	up(&params->lock);

	//PDEBUG(MSG_KUIPARAM, "%s: minOffset: %d, maxOffset: %d, lastOffset: %d, queuedelay: %d\n", DEBUG_NAME, params->minOffset, params->maxOffset, params->lastOffset, queue_delay);

	//atomic_set(&params->QueueDelay, queue_delay);
	
//	return atomic_read(&params->QueueDelay);
	return queue_delay;
}
