/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef PRRT_RELAY_H
#define	PRRT_RELAY_H

#include "prrt_socket.h"

/**
 * This structure represents a relay for two sockets that can forward data from
 * one socket to another.
 * The relay will use an extra thread to pull data from one socket and forward
 * it to the other socket.
 * The relay can only forward data in one direction for the time being. Feel free
 * to implement two way forwarding of data streams.
 */
__PCLASS struct prrt_relay {
    Relayno_t               relayno;                     /*the number of the relay in the procfs*/
    char                    relayname[RELAYNAMELENGTH+1];  /*the name of the relay*/

    struct prrt_socket      *msock_in;          /*the prrt socket for receiving data*/
    struct prrt_socket      *msock_out;         /*the prrt socket for sending data*/
    struct semaphore        socket_lock;	/*locks the variables above*/

    /*variables for relaying data*/
    struct delayed_work     relayData_work;
    struct semaphore        relayData_lock;	/*locks the variables above*/

    struct workqueue_struct *relay_queue;	/**< This thread will forward incoming data packets.*/
    
    enum worker_thread_status_t  relay_status;  /*status of the relaying thread*/
};

/**
 * Allocate a new relay.
 * @param mrelay This variable will hold a pointer to the new relay on success.
 * @return 0 on success, <0 on failure.
 */
int prrt_relay(struct prrt_relay **mrelay, char *relayname);

/**
 * Start the process of relaying data between the two sockets
 * @param mrelay The relay that should be started.
 * @param msock_in The socket that will receive data.
 * @param msock_out The socket that will forward the data.
 * @return 0 on successful start, <0 on failure
 */
int prrt_start_relay(struct prrt_relay *mrelay);

/**
 * Stops the process of relaying data.
 * @param mrelay The relay that should be stopped.
 */
void prrt_shutdown_relay(struct prrt_relay *mrelay);

#endif	/* PRRT_RELAY_H */

