/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "recv_feedback.h"
#include "defines.h"
#include "prrt_socket.h"

#include "kernel_wrappers.h"

#define DEBUG_NAME "RECV_FEEDBACK"

static int recv_feedback(struct prrt_socket *msock);
static int recv_feedback_at_receiver(struct prrt_socket *msock);

void prrt_recv_feedback(struct work_struct *work)
{
	struct prrt_socket *msock = 0;
	struct delayed_work *dw = 0;

	dw = container_of(work, struct delayed_work, work);
	msock = container_of(dw, struct prrt_socket, recvfb_work);

// 	PDEBUG(MSG_COMMONSEND, "%s: Taking spinlock recvfb\n", DEBUG_NAME);
	//down(&msock->recvfb_lock);

// 	PDEBUG(MSG_COMMONSEND, "%s: Spinlock recvfb taken, polling feedback..\n",
// 		DEBUG_NAME);

	/*Receive feedback and take action*/
	for(;;) {
		//PDEBUG(MSG_RECVFBDEB, "%s: Grabbing lock\n", DEBUG_NAME);
		down(&msock->out_lock);
		if(msock->out_status == DOWN) {
			//PDEBUG(MSG_RECVFBDEB, "%s: Leaving loop\n", DEBUG_NAME);
			PDEBUG(MSG_COMMON, "%s: Leaving loop\n", DEBUG_NAME);
			break;
		}
		//PDEBUG(MSG_RECVFBDEB, "%s: Releasing lock\n", DEBUG_NAME);
		up(&msock->out_lock);
		//PDEBUG(MSG_RECVFBDEB, "%s: Starting to wait for feedback\n", DEBUG_NAME);
		recv_feedback(msock);
		//PDEBUG(MSG_RECVFBDEB, "%s: Done receiving feedback\n", DEBUG_NAME);
	}
	//PDEBUG(MSG_RECVFBDEB, "%s: Left loop releasing lock\n", DEBUG_NAME);
	up(&msock->out_lock);
	//PDEBUG(MSG_RECVFBDEB, "%s: Lock released\n", DEBUG_NAME);

// 	queue_delayed_work(msock->recvfb_queue, &msock->recvfb_work,
// 				FEEDBACKDELAY);

	//up(&msock->recvfb_lock);
// 	PDEBUG(MSG_COMMONSEND, "%s: Feedback receiving done\n", DEBUG_NAME);
}

void prrt_recv_feedback_at_receiver(struct work_struct *work)
{
	struct prrt_socket *msock = 0;
	struct delayed_work *dw = 0;

	dw = container_of(work, struct delayed_work, work);
	msock = container_of(dw, struct prrt_socket, recvfb_work);

	/*Receive feedback and take action*/
	for(;;) {
		down(&msock->poll_lock);
		if(msock->poll_status == DOWN) {
      PDEBUG(MSG_COMMON, "%s: leave loop prrt_recv_feedback_at_receiver with max fb sent delay %d (ms)\n", DEBUG_NAME, msock->inc_packet_list.lossgath.max_fb_send_delay);
  //PERROR("%s: leave loop prrt_recv_feedback_at_receiver with max fb sent delay %d (ms)\n", DEBUG_NAME, msock->inc_packet_list.lossgath.max_fb_send_delay);
			break;
		}
		up(&msock->poll_lock);
		recv_feedback_at_receiver(msock);
	}
	up(&msock->poll_lock);
}

static int recv_feedback(struct prrt_socket *msock)
{
	int err = 0;
	struct prrt_fbpacket_container *mfbcont = 0;
	u16 packet_size;
	u16 i = 0;
	u32 receiver;
	int gather_feedback = 0;
	struct prrt_fbblock bl;
	u32 prrtTimeNow = 0;
	u32 sampleRTT = 0;
// 	unsigned int sampleReplace = 0;

	err = prrt_fbpacket_alloc(&mfbcont);
	if(err < 0) {
		PERROR("%s: Unable to receive feedback, could not allocate packet\n", DEBUG_NAME);
		return err;
	}

	//PDEBUG(MSG_RECVFBDEB, "%s: Waiting for feedback packet\n", DEBUG_NAME);
	err = wrap_kernel_recvmsg(msock->sockfb, &(mfbcont->fb), MAX_FB_PACKET_SIZE);
	if(err < 0) {
		//PERROR("%s: Timeout on recv feedback\n", DEBUG_NAME);
		prrt_fbpacket_free(&mfbcont);
		return -1;
	}

	gather_feedback = prrtolist_get_option(&msock->out_packet_list, GATHER_FEEDBACK);
	if(gather_feedback == OFF) {
		goto out;
	}

	packet_size = err;

	if (packet_size > PRRT_HEADER_SIZE + PRRT_FB_HEADER_SIZE)
		mfbcont->payload_size = packet_size - (PRRT_HEADER_SIZE + PRRT_FB_HEADER_SIZE);
	else 
		mfbcont->payload_size = 0;
	//PDEBUG(MSG_RECVFBDEB, "%s: Converting feedback packet to host byte order\n", DEBUG_NAME);
	mfbcont->block_count = mfbcont->payload_size / PRRT_FB_BLOCK_SIZE;
	prrt_fbpacket_ntoh(mfbcont);
	//PDEBUG(MSG_FINAL, "%s: Got Feedback packet: index %u, payload_size %u, receiver_addr %u, rtt_probe %u, packno %u, retr_round %u\n", DEBUG_NAME, mfbcont->fb.header.index, mfbcont->payload_size, mfbcont->fb.fbheader.receiver_addr, mfbcont->fb.fbheader.rtt_probe, mfbcont->fb.blocks.packno, mfbcont->fb.blocks.retr_round);

	receiver = mfbcont->fb.fbheader.receiver_addr;
	//PDEBUG(MSG_RECVFBDEB, "%s: Setting PLR of feedback sender\n", DEBUG_NAME);
	//prrtrlist_setPLR(&msock->out_packet_list.recv_list, receiver, mfbcont->fb.fbheader.PLR);
// 	sampleReplace = mfbcont->fb.fbheader.receiver_addr;
	//TODO: Move to UpdateReplaceTimes in feedback analyzer
	prrtTimeNow = prrt_get_prrttime(&msock->send_time_state);
	sampleRTT = prrt_diff_timestamps(&msock->send_time_state, prrtTimeNow, mfbcont->fb.fbheader.rtt_probe);
	prrtnpar_update_ReplaceTime(&msock->net_params, sampleRTT);
	PDEBUG(MSG_PARAM, "%s: Got RTT probe. Estimated RTT: %d\n", DEBUG_NAME, sampleRTT);
	prrtnpar_set_PLR(&msock->net_params, mfbcont->fb.fbheader.plr);
	prrtnpar_set_NumGaps(&msock->net_params, mfbcont->fb.fbheader.ngap);
	prrtnpar_set_NumBursts(&msock->net_params, mfbcont->fb.fbheader.nburst);
	prrtnpar_set_GapLength(&msock->net_params, mfbcont->fb.fbheader.gap);
	prrtnpar_set_BurstLength(&msock->net_params, mfbcont->fb.fbheader.burst);
	prrtnpar_update_FTT(&msock->net_params, mfbcont->fb.fbheader.ftt);
	//prrtlpar_set_BW_est(&msock->loc_params_out, mfbcont->fb.fbheader.bw_est);
	prrtcpar_set_BW_est(&msock->cod_params, mfbcont->fb.fbheader.bw_est);
	prrtlpar_set_BUF_fb(&msock->loc_params_in, mfbcont->fb.fbheader.buf_fb);
	//PDEBUG(MSG_FINAL, "%s: Got BUF feedback: %d\n", DEBUG_NAME, mfbcont->fb.fbheader.buf_fb);

	PDEBUG(MSG_RECVFBDEB, "%s: Got feedback packet with %d fields:\n", DEBUG_NAME, mfbcont->block_count);
	for(i=0; i<mfbcont->block_count; i++) {
		prrt_fbpacket_get_block(mfbcont, &bl, i);

		//PDEBUG(MSG_RECVFBDEB, "%s: Found number field(pos %d) in fbpacket (no %d)\n", DEBUG_NAME,
		//	i, bl.packno);
		prrtolist_insert_feedback(&msock->out_packet_list, bl.packno, receiver, bl.retr_round);

		//TODO: rather measure at the receiver?
		//Notify network statistics about lost packet
		prrtnpar_insert_Loss(&msock->net_params, bl.packno);
	}

out:
	prrt_fbpacket_free(&mfbcont);
	return 0;
}

//static int update_fb_entry(struct prrt_loss_gatherer *gatherer, struct prrt_fbpacket_container *fb_packet);

static int recv_feedback_at_receiver(struct prrt_socket *msock)
{
	int err = 0;
	struct prrt_fbpacket_container *mfbcont = 0;
  struct prrt_loss_gatherer *loss_gatherer = &(msock->inc_packet_list.lossgath);
  struct fbgath_fb_entry *fbe = 0;
	u16 packet_size = 0;
	u16 i = 0;
	u32 receiver = 0;
	struct prrt_fbblock bl;
	u32 prrtTimeNow = 0;
	u32 sampleRTT = 0;
	struct list_head *iter = 0;
	struct list_head *next = 0;
	u32 max_replace_time = MAX(5, prrtnpar_get_ReplaceTime(&msock->net_params) + WORKDELAYMS);

// 	unsigned int sampleReplace = 0;

	err = prrt_fbpacket_alloc(&mfbcont);
	if(err < 0) {
		PERROR("%s: Unable to receive feedback, could not allocate packet\n", DEBUG_NAME);
		return err;
	}

	//PDEBUG(MSG_RECVFBDEB, "%s: Waiting for feedback packet\n", DEBUG_NAME);
	err = wrap_kernel_recvmsg(msock->sockfb, &(mfbcont->fb), MAX_FB_PACKET_SIZE);
	if(err < 0) {
		//PERROR("%s: Timeout on recv feedback\n", DEBUG_NAME);
		prrt_fbpacket_free(&mfbcont);
		return -1;
	}

	packet_size = err;

	if (packet_size > PRRT_HEADER_SIZE + PRRT_FB_HEADER_SIZE)
		mfbcont->payload_size = packet_size - (PRRT_HEADER_SIZE + PRRT_FB_HEADER_SIZE);
	else 
		mfbcont->payload_size = 0;
	//PDEBUG(MSG_RECVFBDEB, "%s: Converting feedback packet to host byte order\n", DEBUG_NAME);
	mfbcont->block_count = mfbcont->payload_size / PRRT_FB_BLOCK_SIZE;
	prrt_fbpacket_ntoh(mfbcont);
	//PDEBUG(MSG_FINAL, "%s: Got Feedback packet: index %u, payload_size %u, receiver_addr %u, rtt_probe %u, packno %u, retr_round %u\n", DEBUG_NAME, mfbcont->fb.header.index, mfbcont->payload_size, mfbcont->fb.fbheader.receiver_addr, mfbcont->fb.fbheader.rtt_probe, mfbcont->fb.blocks.packno, mfbcont->fb.blocks.retr_round);

	receiver = ntohl(mfbcont->fb.fbheader.receiver_addr);
	//TODO: Move to UpdateReplaceTimes in feedback analyzer
	//prrtTimeNow = prrt_get_prrttime(&msock->send_time_state);
	prrtTimeNow = prrt_get_prrttime(msock->inc_packet_list.time_state);
	sampleRTT = prrt_diff_timestamps(&msock->send_time_state, prrtTimeNow, mfbcont->fb.fbheader.rtt_probe);

	PDEBUG(MSG_RECVFBDEB, "%s: Got feedback packet with %d fields at the receiver.\n", DEBUG_NAME, mfbcont->block_count);
	//PERROR("%s: Got feedback packet with %d fields at the receiver=%lu.\n", DEBUG_NAME, mfbcont->block_count, receiver);
	down(&loss_gatherer->lock);
	for(i=0; i<mfbcont->block_count; i++) {
		prrt_fbpacket_get_block(mfbcont, &bl, i);

    // We initiate to check if the feedback entry in loss gatherer is due to send or is removed.
    list_for_each_safe(iter, next, &loss_gatherer->loss_list) {
      fbe = container_of(iter, struct fbgath_fb_entry, list);

      if(fbe->miss_packno == bl.packno && fbe->retr_round == bl.retr_round) {
        /*found the same feedback entry.*/
        //prrtTimeNow = prrt_get_prrttime(msock->inc_packet_list.time_state);
        PERROR("%s: Found feedback entry for packet with packetno=%u and blockno=%u at the receiver now=%u and next_send=%u\n", DEBUG_NAME, (int)bl.packno, i, prrtTimeNow, fbe->last_send + max_replace_time + fbe->fb_send_delay);
        if (prrtTimeNow <= fbe->last_send + max_replace_time + fbe->fb_send_delay) {
          PDEBUG(MSG_RECVFBDEB, "%s: Updating feedback entry for packet with packetno=%d at the receiver\n", DEBUG_NAME, (int)bl.packno);
          //PERROR("%s: Updating feedback entry for packet with packetno=%d at the receiver\n", DEBUG_NAME, (int)bl.packno);
          /*update the feedback entry.*/
          fbe->last_send += max_replace_time;
          fbe->retr_round++;
        }
        //break;
      }
    }
    //PERROR("%s: Searched feedback entry for packet with packetno=%d and blockno=%d at the receiver\n", DEBUG_NAME, (int)bl.packno, i);

	}
	up(&loss_gatherer->lock);

	prrt_fbpacket_free(&mfbcont);
	return 0;
}
