/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "recv_feedback.h"
#include "defines.h"
#include "prrt_socket.h"

#define DEBUG_NAME "SEND_REPLACEMENTS"

static int send_replacements(struct prrt_socket *msock);

void prrt_send_replacements(struct work_struct *work)
{
	struct prrt_socket *msock = 0;
	struct delayed_work *dw = 0;
	enum worker_thread_status_t status;

	dw = container_of(work, struct delayed_work, work);
	msock = container_of(dw, struct prrt_socket, sendrepl_work);

	down(&msock->poll_lock);
	status = msock->out_status;
	up(&msock->poll_lock);

	if(status == DOWN)
		return;

	send_replacements(msock);

	
	queue_delayed_work(msock->sendrepl_queue, &msock->sendrepl_work, REPLACEMENTDELAY);
}

static int send_replacements(struct prrt_socket *msock)
{
	int err = 0;
	//int send_tokens = 0;

	prrtolist_send_replacements(&msock->out_packet_list, msock->sockfb, msock->peer);

	/*send all the redundancy that is available for the 0th round*/
	while (err >= 0) {
		err = prrtolist_try_to_send_redundancy_of_next_coded_block(&msock->out_packet_list,
				msock->sock, msock->peer);
	}
	//PDEBUG(MSG_SENDFBDEB, "%s: Sending of pre redundancy done\n", DEBUG_NAME);

	return 0;
}
