/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "list_stats.h"

#define DEBUG_NAME "LIST_STATS"

void prrtstats_init_list_stats(struct prrt_list_stats *mlstats)
{
	init_MUTEX(&mlstats->lock);

	down(&mlstats->lock);

	mlstats->av_list_data = 0;
	mlstats->av_list_packs = 0;

	mlstats->max_list_data = 0;
	mlstats->max_list_packs = 0;

	mlstats->min_list_data = 429496729;
	mlstats->min_list_packs = 429496729;

	mlstats->count_data = 0;
	mlstats->count_packs = 0;

	up(&mlstats->lock);
}

void prrtstats_reinit_list_stats(struct prrt_list_stats *mlstats)
{
	down(&mlstats->lock);

	mlstats->av_list_data = 0;
	mlstats->av_list_packs = 0;

	mlstats->max_list_data = 0;
	mlstats->max_list_packs = 0;

	mlstats->min_list_data = 429496729;
	mlstats->min_list_packs = 429496729;

	mlstats->count_data = 0;
	mlstats->count_packs = 0;

	up(&mlstats->lock);
}

void prrtstats_update_list_stats_add(struct prrt_list_stats *mlstats, struct prrt_packet_container *pcont)
{
	down(&mlstats->lock);

	mlstats->count_data += pcont->payload_size;
	mlstats->count_packs++;

	/*TODO: average value estimation*/
	PDEBUG(MSG_STATS, "%s: average data >> 1 = %lu\n", DEBUG_NAME, mlstats->av_list_data >> 1);
	PDEBUG(MSG_STATS, "%s: absolute data >> 1 = %lu\n", DEBUG_NAME, mlstats->count_data >> 1);
	mlstats->av_list_data = (mlstats->av_list_data >> 1) + (mlstats->count_data >> 1);
	PDEBUG(MSG_STATS, "%s: average data new = %lu\n", DEBUG_NAME, mlstats->av_list_data);
// 	pack_count++;
// 	if(pack_count == 10) {
// 		mlstats->av_list_packs += 1;
// 	}

	if(mlstats->count_data > mlstats->max_list_data)
		mlstats->max_list_data = mlstats->count_data;
	else if(mlstats->count_data < mlstats->min_list_data)
		mlstats->min_list_data = mlstats->count_data;

	if(mlstats->count_packs > mlstats->max_list_packs)
		mlstats->max_list_packs = mlstats->count_packs;
	else if(mlstats->count_packs < mlstats->min_list_packs)
		mlstats->min_list_packs = mlstats->count_packs;

	up(&mlstats->lock);
}

void prrtstats_update_list_stats_del(struct prrt_list_stats *mlstats, struct prrt_packet_container *pcont)
{
	down(&mlstats->lock);

	mlstats->count_data -= pcont->payload_size;
	mlstats->count_packs--;

	/*TODO: average value estimation*/
// 	mlstats->av_list_data = (mlstats->av_list_data >> 1) + (mlstats->count_data >> 1);
//  	mlstats->av_list_data -= pcont->pack.data.header.payload_size >> 2;
// 	pack_count++;
// 	if(pack_count == 10) {
// 		mlstats->av_list_packs += 1;
// 	}

	if(mlstats->count_data > mlstats->max_list_data)
		mlstats->max_list_data = mlstats->count_data;
	else if(mlstats->count_data < mlstats->min_list_data)
		mlstats->min_list_data = mlstats->count_data;

	if(mlstats->count_packs > mlstats->max_list_packs)
		mlstats->max_list_packs = mlstats->count_packs;
	else if(mlstats->count_packs < mlstats->min_list_packs)
		mlstats->min_list_packs = mlstats->count_packs;

	up(&mlstats->lock);
}

unsigned long prrtstats_get_av_list_data(struct prrt_list_stats *mlstats)
{
	unsigned long ret = 0;

	down(&mlstats->lock);

	ret = mlstats->av_list_data;

	up(&mlstats->lock);

	return ret;
}

unsigned long prrtstats_get_av_list_packs(struct prrt_list_stats *mlstats)
{
	unsigned long ret = 0;

	down(&mlstats->lock);

	ret = mlstats->av_list_packs;

	up(&mlstats->lock);

	return ret;
}

unsigned long prrtstats_get_max_list_data(struct prrt_list_stats *mlstats)
{
	unsigned long ret = 0;

	down(&mlstats->lock);

	ret = mlstats->max_list_data;

	up(&mlstats->lock);

	return ret;
}

unsigned long prrtstats_get_max_list_packs(struct prrt_list_stats *mlstats)
{
	unsigned long ret = 0;

	down(&mlstats->lock);

	ret = mlstats->max_list_packs;

	up(&mlstats->lock);

	return ret;
}

unsigned long prrtstats_get_min_list_data(struct prrt_list_stats *mlstats)
{
	unsigned long ret = 0;

	down(&mlstats->lock);

	ret = mlstats->min_list_data;

	up(&mlstats->lock);

	return ret;
}

unsigned long prrtstats_get_min_list_packs(struct prrt_list_stats *mlstats)
{
	unsigned long ret = 0;

	down(&mlstats->lock);

	ret = mlstats->min_list_packs;

	up(&mlstats->lock);

	return ret;
}

void prrtstats_print_list_stats(struct prrt_list_stats *mlstats)
{
	unsigned long av_data = 0;
	unsigned long av_packs = 0;
	unsigned long max_data = 0;
	unsigned long min_data = 0;
	unsigned long max_packs = 0;
	unsigned long min_packs = 0;

	down(&mlstats->lock);

	av_data = mlstats->av_list_data;
	av_packs = mlstats->av_list_packs;
	max_data = mlstats->max_list_data;
	min_data = mlstats->min_list_data;
	max_packs = mlstats->max_list_packs;
	min_packs = mlstats->min_list_packs;

	up(&mlstats->lock);

	PDEBUG(MSG_FINAL, "PRRT_SOCK: * Average list size - %lu bytes\n", av_data);
	PDEBUG(MSG_FINAL, "PRRT_SOCK: * Average list length - %lu\n", av_packs);
	PDEBUG(MSG_FINAL, "PRRT_SOCK: * Maximum list size - %lu bytes\n", max_data);
	PDEBUG(MSG_FINAL, "PRRT_SOCK: * Maximum list length - %lu\n", max_packs);
// 	PDEBUG(MSG_FINAL, "PRRT_SOCK: * Minimum list size - %lu bytes\n", min_data);
// 	PDEBUG(MSG_FINAL, "PRRT_SOCK: * Minimum list length - %lu\n", min_packs);
}
