/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "strhelp.h"
#include <linux/string.h>
#include <linux/ctype.h>
#include <linux/slab.h>

/*from below*/
static void strhelp_strip(char *str, int strlen, char **stripped, int *striplen);

/*
 * This function can only handle input strings with single spaces between words.
 * TODO: extend for multiple spaces
 */
int strhelp_create(struct words **wor, char *str, int strlen)
{
	int i = 0;
	int restlen = 0;
	char *c = 0;
	char *c_last = 0;
	char *stripped = 0;
	int striplen = 0;
	struct words *w = 0;

	w = kmalloc(sizeof(*w), GFP_KERNEL);
	if(!w)
		goto nomem;

	strhelp_strip(str, strlen, &stripped, &striplen);

	w->num = 0;

	//printk(KERN_ALERT "Striplen: %d\n", striplen);
	if(striplen == 0)
		goto out;

	w->pos[0] = stripped;
	c_last = stripped;
	restlen = striplen;
	
	while((c = strnchr(w->pos[i], restlen, ' ')) && i<MAXARGS) {
		/*alloc memory for last word and copy it*/
		w->len[i] = c-w->pos[i]+1;
		w->pos[i] = kmalloc(w->len[i], GFP_KERNEL);
		if(!w->pos[i])
			goto nomem;
		strlcpy(w->pos[i], c_last, w->len[i]);
		w->num++;

		/*move pointers*/
		i++;
		w->pos[i] = c+1;
		c_last = c+1;
		restlen = striplen - (w->pos[i]-stripped);
	}

	if(c != 0)
		return -1;

	if(i>=MAXARGS)
		goto nomem;

	/*the last word must be copied separately*/
	w->len[i] = restlen+2;
	w->pos[i] = kmalloc(w->len[i], GFP_KERNEL);
	if(!w->pos[i])
		goto nomem;
	strlcpy(w->pos[i], c_last, w->len[i]);
	w->num++;

out:
	*wor = w;

	return 0;

nomem:
	strhelp_destroy(w);
	return -ENOMEM;
}

void strhelp_destroy(struct words *w)
{
	int i;

	if(!w)
		return;

	for(i=0; i<w->num; i++) {
		kfree(w->pos[i]);
	}

	kfree(w);

	w = 0;
}

/*
 * Computes a pointer to the first non space char and the number of chars
 * until all other chars are spaces or \0.
 */
static void strhelp_strip(char *str, int strlen, char **stripped, int *striplen)
{
	int i = strlen-1;
	*stripped = str;
	*striplen = 0;

	if(str[0] == '\0')
		return;

	/*move stripped pointer to the first non space char*/
	while(isspace(*stripped[0])) {
		*stripped = *stripped+1;
	}

	/*set length of this string*/
	*striplen = strlen - (*stripped-str);
	
	/*decrease length of the stripped string by one for each trailing
	 *space*/
	while(isspace(str[i]) || str[i] == '\0') {
		(*striplen)--;
		i--;
	}
}

