/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include <signal.h>
#include <stdio.h>
#include "socket_list.h"

void prrt_uninit_handler();
void prrt_uninit();

static char initialised;	/*is the prrt lib initialised?(no=0)*/

/**
 * This will set the signal handlers to the uninit handler and initialise
 * the prrt lib.
 */
void prrt_init()
{
	initialised = 1;
	signal(SIGABRT, &prrt_uninit_handler);
	signal(SIGTERM, &prrt_uninit_handler);
	signal(SIGINT, &prrt_uninit_handler);
}

/**
 * Uninitialise the prrt lib.
 */
void prrt_uninit()
{
	int sockfd = 0;
	while(!socket_list_empty()) {
		sockfd = socket_list_del_rnd();
		prrt_close(sockfd);
		printf("Deleted sockfd no %d\n", sockfd);
	}
	printf("ByeBye\n");
}

/**
 * Calls the function to uninit the prrt lib and terminates the program.
 */
void prrt_uninit_handler()
{
	prrt_uninit();
	signal(SIGABRT, SIG_DFL);
	signal(SIGTERM, SIG_DFL);
	signal(SIGINT, SIG_DFL);
	raise(SIGTERM);
}
