/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "socket_list.h"
#include <stdlib.h>

static struct socket_list *sock_list;

int socket_list_add(int sockfd)
{
	struct socket_list *sl = 0;

	/*init socket list if empty or not yet initialised*/
	if(socket_list_empty()) {
		sock_list = malloc(sizeof(*sock_list));
		if(!sock_list)
			return -1;

		sock_list->next = sock_list;
		sock_list->prev = sock_list;
		sock_list->socketno = sockfd;
		return 0;
	}

	sl = malloc(sizeof(*sl));
	if(!sl)
		return -1;

	/*insert new socket at the end of the list*/
	sl->next = sock_list;
	sl->prev = sock_list->prev;
	sl->socketno = sockfd;

	sock_list->prev->next = sl;
	sock_list->prev = sl;

	return 0;
}

int socket_list_del_rnd()
{
	int sockfd = 0;
	struct socket_list *sl = 0;

	/*is the list empty?*/
	if(socket_list_empty())
		return -1;

	/*is there only one element left?*/
	if(sock_list->next == sock_list && sock_list->prev == sock_list) {
		sockfd = sock_list->socketno;
		free(sock_list);
		sock_list = 0;
		return sockfd;
	}

	/*get first element of the list*/
	sl = sock_list->next;

	sock_list->next = sl->next;
	sl->next->prev = sock_list;

	sockfd = sl->socketno;
	free(sl);

	return sockfd;
}

int socket_list_del(int sockfd)
{
	struct socket_list *sl = 0;

	/*is the list empty?*/
	if(socket_list_empty())
		return -1;

	/*is there only one element left?*/
	if(sock_list->next == sock_list && sock_list->prev == sock_list) {
		if(sockfd == sock_list->socketno) {
			free(sock_list);
			sock_list = 0;
			return 1;
		} else {
			return 0; /*not found*/
		}
	}
	
	/*sockfd is the first entry in list*/
	if(sock_list->socketno == sockfd) {
		sock_list->prev->next = sock_list->next;
		sock_list->next->prev = sock_list->prev;
		sl = sock_list;
		sock_list = sock_list->next;
		free(sl);
		return 1;
	}

	/*sockfd is not the first entry, search for it*/
	for(sl=sock_list->next; sl!=sock_list; sl=sl->next) {
		if(sl->socketno == sockfd) {
			sl->prev->next = sl->next;
			sl->next->prev = sl->prev;
			free(sl);
			return 1;
		}
	}

	return 0; /*not found*/
}

inline int socket_list_empty()
{
	return (sock_list == 0);
}
